-------------------------------------------------------------------------------
--
-- Title       : shift_chain
-- Design      : tcl
-- Author      : Vegard
-- Company     : NTNU
-- Description : 
-------------------------------------------------------------------------------

library IEEE;
use IEEE.STD_LOGIC_1164.all;

entity shift_chain is
   generic(											  
		C_SIZE	 	: natural := 8; -- size of each register
		C_NUM_REG 	: natural := 4	-- number of registers
	);
	port(	   
		clk		: in  std_logic;
		sel		: in  std_logic;
		shift	: in  std_logic;
		write	: in  std_logic_vector(C_NUM_REG-1 downto 0);
		data_i	: in  std_logic_vector(C_SIZE*C_NUM_REG -1 downto 0); 
		shift_i : in  std_logic_vector(C_SIZE-1 downto 0);
		data_o	: out std_logic_vector(C_SIZE*C_NUM_REG -1 downto 0);
		shift_o	: out std_logic_vector(C_SIZE-1 downto 0)
	);	

end shift_chain;


architecture shift_chain of shift_chain is

signal shift_l 		: std_logic;

signal shift_il	: std_logic_vector(C_SIZE*C_NUM_REG-1 downto 0);
signal shift_ol	:std_logic_vector(C_SIZE*C_NUM_REG-1 downto 0);

begin
	
	shift_l <= shift and sel;
	
	SHIFT_PROC0 : process(shift_ol,shift_il,shift_i) is
	begin		  
		shift_il(C_SIZE-1 downto 0) <= shift_i;
		for i in 1 to C_NUM_REG-1 loop
			shift_il((i+1)*C_SIZE-1 downto (i)*C_SIZE) <= shift_ol((i)*C_SIZE-1 downto (i-1)*C_SIZE);
		end loop;
		shift_o <= shift_ol(C_SIZE*C_NUM_REG-1 downto C_SIZE*(C_NUM_REG-1));
	end process SHIFT_PROC0;
	
	REG_GEN0: for i in 0 to C_NUM_REG-1 generate
		DATA_OUT : entity register_macro
		generic map(
			C_SIZE => C_SIZE,
			C_NUM_REG=> 1
		)
		port map(
		    clk 	=> clk, 
			data_i	=> data_i((i+1)*C_SIZE-1 downto (i)*C_SIZE),
			write 	=> write(i),
			shift 	=> shift_l,
			shift_i => shift_il((i+1)*C_SIZE-1 downto (i)*C_SIZE),	
			shift_o => shift_ol((i+1)*C_SIZE-1 downto (i)*C_SIZE),
			data_o	 =>  data_o((i+1)*C_SIZE-1 downto (i)*C_SIZE)	
		);
	end generate;

end shift_chain;
