-------------------------------------------------------------------------------
--
-- Title       : register_block
-- Design      : ps_registers
-- Author      : Vegard Endresen
-- Company     : NTNU
-- Description : Register block	for parallel shift macro.
-------------------------------------------------------------------------------

library IEEE;
use IEEE.STD_LOGIC_1164.all;

entity register_block is
	generic(C_SIZE : natural :=8);
	port(	   
		clk		: in  std_logic;
		reset	: in  std_logic;
		write	: in  std_logic;
		data_i  : in  std_logic_vector(C_SIZE-1 downto 0);
		data_o	: out std_logic_vector(C_SIZE-1 downto 0)
	);

end register_block;


architecture RTL of register_block is

signal memory : std_logic_vector(C_SIZE-1 downto 0);
begin
	
	data_o <= memory;
	write_proc : process(clk) is 
	
	begin  
		if clk'event and clk ='1' then
			if reset ='1' then
				memory <= (others =>'0');
			elsif write = '1' then 
				memory <= data_i;
			end if;
		end if;
		
	end process write_proc;
	
end RTL;
