------------------------------------------------------------------------------
-- user_logic.vhd - entity/architecture pair
------------------------------------------------------------------------------
--
-- ***************************************************************************
-- ** Copyright (c) 1995-2008 Xilinx, Inc.  All rights reserved.            **
-- **                                                                       **
-- ** Xilinx, Inc.                                                          **
-- ** XILINX IS PROVIDING THIS DESIGN, CODE, OR INFORMATION "AS IS"         **
-- ** AS A COURTESY TO YOU, SOLELY FOR USE IN DEVELOPING PROGRAMS AND       **
-- ** SOLUTIONS FOR XILINX DEVICES.  BY PROVIDING THIS DESIGN, CODE,        **
-- ** OR INFORMATION AS ONE POSSIBLE IMPLEMENTATION OF THIS FEATURE,        **
-- ** APPLICATION OR STANDARD, XILINX IS MAKING NO REPRESENTATION           **
-- ** THAT THIS IMPLEMENTATION IS FREE FROM ANY CLAIMS OF INFRINGEMENT,     **
-- ** AND YOU ARE RESPONSIBLE FOR OBTAINING ANY RIGHTS YOU MAY REQUIRE      **
-- ** FOR YOUR IMPLEMENTATION.  XILINX EXPRESSLY DISCLAIMS ANY              **
-- ** WARRANTY WHATSOEVER WITH RESPECT TO THE ADEQUACY OF THE               **
-- ** IMPLEMENTATION, INCLUDING BUT NOT LIMITED TO ANY WARRANTIES OR        **
-- ** REPRESENTATIONS THAT THIS IMPLEMENTATION IS FREE FROM CLAIMS OF       **
-- ** INFRINGEMENT, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS       **
-- ** FOR A PARTICULAR PURPOSE.                                             **
-- **                                                                       **
-- ***************************************************************************
--
------------------------------------------------------------------------------
-- Filename:          user_logic.vhd
-- Version:           1.00.a
-- Description:       User logic.
-- Date:              Fri Apr 09 10:27:18 2010 (by Create and Import Peripheral Wizard)
-- VHDL Standard:     VHDL'93
------------------------------------------------------------------------------
-- Naming Conventions:
--   active low signals:                    "*_n"
--   clock signals:                         "clk", "clk_div#", "clk_#x"
--   reset signals:                         "rst", "rst_n"
--   generics:                              "C_*"
--   user defined types:                    "*_TYPE"
--   state machine next state:              "*_ns"
--   state machine current state:           "*_cs"
--   combinatorial signals:                 "*_com"
--   pipelined or register delay signals:   "*_d#"
--   counter signals:                       "*cnt*"
--   clock enable signals:                  "*_ce"
--   internal version of output port:       "*_i"
--   device pins:                           "*_pin"
--   ports:                                 "- Names begin with Uppercase"
--   processes:                             "*_PROCESS"
--   component instantiations:              "<ENTITY_>I_<#|FUNC>"
------------------------------------------------------------------------------

-- DO NOT EDIT BELOW THIS LINE --------------------
library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_unsigned.all;



-- DO NOT EDIT ABOVE THIS LINE --------------------

--USER libraries added here

------------------------------------------------------------------------------
-- Entity section
------------------------------------------------------------------------------
-- Definition of Generics:
--   C_SLV_AWIDTH                 -- Slave interface address bus width
--   C_SLV_DWIDTH                 -- Slave interface data bus width
--   C_NUM_REG                    -- Number of software accessible registers
--   C_NUM_MEM                    -- Number of memory spaces
--
-- Definition of Ports:
--   Bus2IP_Clk                   -- Bus to IP clock
--   Bus2IP_Reset                 -- Bus to IP reset
--   Bus2IP_Addr                  -- Bus to IP address bus
--   Bus2IP_CS                    -- Bus to IP chip select for user logic memory selection
--   Bus2IP_RNW                   -- Bus to IP read/not write
--   Bus2IP_Data                  -- Bus to IP data bus
--   Bus2IP_BE                    -- Bus to IP byte enables
--   Bus2IP_RdCE                  -- Bus to IP read chip enable
--   Bus2IP_WrCE                  -- Bus to IP write chip enable
--   IP2Bus_Data                  -- IP to Bus data bus
--   IP2Bus_RdAck                 -- IP to Bus read transfer acknowledgement
--   IP2Bus_WrAck                 -- IP to Bus write transfer acknowledgement
--   IP2Bus_Error                 -- IP to Bus error response
------------------------------------------------------------------------------

entity user_logic is
  generic
  (
    -- ADD USER GENERICS BELOW THIS LINE ---------------
    --USER generics added here
    -- ADD USER GENERICS ABOVE THIS LINE ---------------

    -- DO NOT EDIT BELOW THIS LINE ---------------------
    -- Bus protocol parameters, do not add to or delete
    C_SLV_AWIDTH                   : integer              := 32;
    C_SLV_DWIDTH                   : integer              := 32;
    C_NUM_REG                      : integer              := 2;
    C_NUM_MEM                      : integer              := 1;
	 INSTR_SIZE							  : natural				    := 16; 
    ADDR_BITS 							  : natural 				 :=3	
    -- DO NOT EDIT ABOVE THIS LINE ---------------------
  );
  port
  (
    -- ADD USER PORTS BELOW THIS LINE ------------------
    --USER ports added here
    -- ADD USER PORTS ABOVE THIS LINE ------------------

    -- DO NOT EDIT BELOW THIS LINE ---------------------
    -- Bus protocol ports, do not add to or delete
    Bus2IP_Clk                     : in  std_logic;
    Bus2IP_Reset                   : in  std_logic;
    Bus2IP_Addr                    : in  std_logic_vector(0 to C_SLV_AWIDTH-1);
    Bus2IP_CS                      : in  std_logic_vector(0 to C_NUM_MEM-1);
    Bus2IP_RNW                     : in  std_logic;
    Bus2IP_Data                    : in  std_logic_vector(0 to C_SLV_DWIDTH-1);
    Bus2IP_BE                      : in  std_logic_vector(0 to C_SLV_DWIDTH/8-1);
    Bus2IP_RdCE                    : in  std_logic_vector(0 to C_NUM_REG-1);
    Bus2IP_WrCE                    : in  std_logic_vector(0 to C_NUM_REG-1);
    IP2Bus_Data                    : out std_logic_vector(0 to C_SLV_DWIDTH-1);
    IP2Bus_RdAck                   : out std_logic;
    IP2Bus_WrAck                   : out std_logic;
    IP2Bus_Error                   : out std_logic
    -- DO NOT EDIT ABOVE THIS LINE ---------------------
  );

  attribute SIGIS : string;
  attribute SIGIS of Bus2IP_Clk    : signal is "CLK";
  attribute SIGIS of Bus2IP_Reset  : signal is "RST";

end entity user_logic;

------------------------------------------------------------------------------
-- Architecture section
------------------------------------------------------------------------------

architecture IMP of user_logic is
 
  ---------------------------------------------------------------------
  -- HW - BRAM signals
	signal bram_hw_en			:  std_logic;
	signal bram_hw_we   		:  std_logic;
	signal bram_hw_addr			:  std_logic_vector(7 downto 0);
	signal bram_hw_data_i 		:  std_logic_vector(C_SLV_DWIDTH-1 downto 0);
	signal bram_hw_data_o 		:  std_logic_vector(C_SLV_DWIDTH-1 downto 0);
	---------------------------------------------------------------------		
  
  
  ---------------------------------------------------------------------
  -- Shift controller signals
  signal shift			: std_logic;
  ---------------------------------------------------------------------		
  signal if_instr_write : std_logic;
  ---------------------------------------------------------------------
  -- Instruction controller signals
  
  
	signal id_en : std_logic;
	signal id_we : std_logic;
	signal id_read_ack : std_logic;
  
	signal instruction 		: std_logic_vector(INSTR_SIZE-1 downto 0);
	signal id_addr		    : std_logic_vector(7 downto 0);
	signal register_write	: std_logic;
	signal count			: std_logic_vector(1 downto 0);
	signal shift_mask		: std_logic;
	signal zero_mask		: std_logic;	
	signal instr_complete   : std_logic;
  ---------------------------------------------------------------------
  
  
  
  
  ---------------------------------------------------------------------
  -- REG0 signals
  
  signal reg0_shift_o : std_logic_vector(0 to 7);
  signal reg0_data_o : std_logic_vector(31 downto 0);
  ---------------------------------------------------------------------
  
  
  ---------------------------------------------------------------------
  -- Multiplier signals
  signal hw_complete : std_logic;
  signal hw_run : std_logic;

  signal instr_write : std_logic;
  signal zeros : std_logic_vector(31 downto 0); 

  
  signal id_write_ack :std_logic;
  signal mult_shift_o : std_logic_vector(7 downto 0);


  ------------------------------------------
  -- Signals for user logic slave model s/w accessible register example
  ------------------------------------------
  signal slv_reg0                       : std_logic_vector(0 to C_SLV_DWIDTH-1);
  signal slv_reg1                       : std_logic_vector(0 to C_SLV_DWIDTH-1);
  signal slv_reg_write_sel              : std_logic_vector(0 to 1);
  signal slv_reg_read_sel               : std_logic_vector(0 to 1);
  signal slv_ip2bus_data                : std_logic_vector(0 to C_SLV_DWIDTH-1);
  signal slv_read_ack                   : std_logic;
  signal slv_write_ack                  : std_logic;

  ------------------------------------------
  -- Signals for user logic memory space example
  ------------------------------------------
  signal mem_read_ack               : std_logic;
  signal mem_write_ack              : std_logic;
  signal mem_ip2bus_error			   : std_logic;
  signal mem_ip2bus_data            : std_logic_vector(0 to C_SLV_DWIDTH-1);
			

begin

  --USER logic implementation added here

  ------------------------------------------
  -- Example code to read/write user logic slave model s/w accessible registers
  -- 
  -- Note:
  -- The example code presented here is to show you one way of reading/writing
  -- software accessible registers implemented in the user logic slave model.
  -- Each bit of the Bus2IP_WrCE/Bus2IP_RdCE signals is configured to correspond
  -- to one software accessible register by the top level template. For example,
  -- if you have four 32 bit software accessible registers in the user logic,
  -- you are basically operating on the following memory mapped registers:
  -- 
  --    Bus2IP_WrCE/Bus2IP_RdCE   Memory Mapped Register
  --                     "1000"   C_BASEADDR + 0x0
  --                     "0100"   C_BASEADDR + 0x4
  --                     "0010"   C_BASEADDR + 0x8
  --                     "0001"   C_BASEADDR + 0xC
  -- 
  ------------------------------------------
  slv_reg_write_sel <= Bus2IP_WrCE(0 to 1);
  slv_reg_read_sel  <= Bus2IP_RdCE(0 to 1);
  slv_write_ack     <= Bus2IP_WrCE(0) or Bus2IP_WrCE(1);
  slv_read_ack      <= Bus2IP_RdCE(0) or Bus2IP_RdCE(1);

  -- implement slave model software accessible register(s)
  SLAVE_REG_WRITE_PROC : process( Bus2IP_Clk ) is
  begin

    if Bus2IP_Clk'event and Bus2IP_Clk = '1' then
      if Bus2IP_Reset = '1' then
        slv_reg0 <= (others => '0');
      else	
	  	slv_reg0(31) <= instr_complete;
		slv_reg0(24 to 30) <=(others =>'0');
        case slv_reg_write_sel is
        	when "10" =>
            for byte_index in 0 to 2 loop
              if ( Bus2IP_BE(byte_index) = '1' ) then
                slv_reg0(byte_index*8 to byte_index*8+7) <= Bus2IP_Data(byte_index*8 to byte_index*8+7);
              end if;
            end loop;
          when others => null;
        end case;
      end if;
    end if;

  end process SLAVE_REG_WRITE_PROC;



  -- implement slave model software accessible register(s) read mux
  SLAVE_REG_READ_PROC : process( slv_reg_read_sel, slv_reg0 ) is
  begin

    case slv_reg_read_sel is
      when "10" => slv_ip2bus_data <= slv_reg0;
		when "01" => slv_ip2bus_data <= slv_reg1;
      when others => slv_ip2bus_data <= (others => '0');
    end case;

  end process SLAVE_REG_READ_PROC;

  
  zeros <= (others =>'0');
	
	
  TIMER_PROC : process(Bus2IP_Clk) is
  begin
	  if Bus2IP_Clk'event and Bus2IP_Clk ='1' then
	     if slv_reg0(16) ='1' then
			 slv_reg1 <= (others =>'0');
	     elsif slv_reg0(17) ='1' then
			 slv_reg1 <=  slv_reg1 +1;
		 else
			slv_reg1 <= slv_reg1;
		end if;
	  end if;
  end process TIMER_PROC;
  
	
	
	BRAM_UNIT : entity bram_packed 
	port map(
      hw_clk 		=> Bus2IP_Clk,
		hw_en		 => bram_hw_en,
		hw_we   	 => bram_hw_we,
		hw_addr	 	=> bram_hw_addr,
		hw_data_i  =>	bram_hw_data_i,
		hw_data_o =>	bram_hw_data_o,
		hw_read_ack => id_read_ack,
		hw_write_ack => id_write_ack,
		
		Bus2IP_Clk                     => Bus2IP_Clk,
      Bus2IP_Reset                   => Bus2IP_Reset,
		Bus2IP_Addr                    => Bus2IP_Addr,
		Bus2IP_CS                      => Bus2IP_CS,
		Bus2IP_RNW                     => Bus2IP_RNW,
		Bus2IP_Data                    => Bus2IP_Data,
		Bus2IP_BE                      => Bus2IP_BE,
      IP2Bus_Data                    => mem_ip2bus_Data,
      IP2Bus_RdAck                   => mem_read_ack,
      IP2Bus_WrAck                   => mem_write_ack,
      IP2Bus_Error                   => mem_ip2bus_error
    );
	

	ICTRL : entity instr_controller
		generic map(
		 	INSTR_SIZE => INSTR_SIZE
		)
		port map(
			clk => Bus2IP_Clk,
			cs => Bus2IP_CS,
			instruction => slv_reg0(0 to 15),
			read_ack => id_read_ack,
			write_ack => id_write_ack,
			hw_complete => hw_complete,
			address => id_addr,
			shift => shift,
			register_write => register_write,
			hw_en => id_en,
			hw_we => id_we,
			shift_mask => shift_mask,
			zero_mask => zero_mask,
			hw_run => hw_run,
			instr_complete => instr_complete
	);
	
	REG0 : entity register_macro
		generic map(
			C_SIZE 		=> 8,
			C_NUM_REG  => 4
		)
		port map(
			clk =>Bus2IP_Clk,
			reset => Bus2IP_Reset,
			shift => shift,
			write => register_write,
			shift_i => zeros(7 downto 0),
			data_i => bram_hw_data_o,
			data_o => REG0_data_o,  -- open
			shift_o	=> REG0_shift_o
		);
		
		
	REG1 : entity register_macro
		generic map(
			C_SIZE 		=> 8,
			C_NUM_REG  => 4
		)
		port map(
			clk =>Bus2IP_Clk,
			reset => Bus2IP_Reset,
			shift => shift,
			shift_i => mult_shift_o,
			write   => zeros(0),
			shift_o => open,
			data_i => zeros,
			data_o => bram_hw_data_i
		);		  
		
	MULT : entity multiplier
	generic map(
		OPERAND_SIZE => 32,
		LOG_OPERAND_SIZE => 5
	)
	port map(
		clk => Bus2IP_Clk,		
		reset => Bus2IP_Reset,
		addr => id_addr(0),
		zerofill => zero_mask,
		shift_i => REG0_shift_o,
		shift_o => mult_shift_o,
		shift_g => shift,
		run 	=> hw_run,
		finish  => hw_complete		
	);

		
		

		
	-- Multiplexing instruction fetch and data operations on BRAM
	
	
	
	bram_hw_addr <= id_addr;
	bram_hw_en <= id_en;
	bram_hw_we <= id_we;
  
  
  ------------------------------------------
  -- Example code to drive IP to Bus signals
  ------------------------------------------
  IP2Bus_Data  <= slv_ip2bus_data when slv_read_ack = '1' else
                  mem_ip2bus_data when mem_read_ack = '1' else
                  (others => '0');

  IP2Bus_WrAck <= slv_write_ack or mem_write_ack;
  IP2Bus_RdAck <= slv_read_ack or mem_read_ack;
  IP2Bus_Error <= '0' or mem_ip2bus_error;

end IMP;
