-------------------------------------------------------------------------------
--
-- Title       : serial_controller
-- Design      : shift_chain
-- Author      : Vegard
-- Company     : NTNU
-- Description : Controller for serial data transfer
-------------------------------------------------------------------------------


library IEEE;
use IEEE.STD_LOGIC_1164.all;
use IEEE.std_logic_unsigned.all;

entity serial_controller is
	port(
		signal clk 					: in  std_logic;	
		signal instruction			: in std_logic_vector(7 downto 0);
		signal HW_complete			: in  std_logic;
		signal shift 				: out std_logic; 
		signal controller_waiting	: out std_logic; 
		signal run					: out std_logic;
		signal zerofill				: out  std_logic;
		signal address				: out  std_logic_vector(1 downto 0)
	);
	
end serial_controller;


architecture serial_controller of serial_controller is

type STATE is (IDLE,EXECUTING,WAITING);

signal cstate,nstate : STATE;
signal start 			: std_logic;
signal shift_complete 	: std_logic; 

-- instruction decomposition
signal CPU_interrupt		:  std_logic; 
signal CPU_opcode			:  std_logic; 
signal CPU_sync				: 	std_logic;




signal instr_complete : std_logic;

begin			
	
	CPU_opcode 		<= instruction(7);	
	CPU_sync 		<= instruction(6);	
	CPU_interrupt 	<= instruction(5);
	zerofill	<= instruction(4);
	address 		<= instruction(3 downto 2);
	instr_complete <= shift_complete or HW_complete;
 
	
	
	NSTATE_PROC : process(cstate,CPU_interrupt,CPU_sync,shift_complete,HW_complete) is
	begin			   
		case cstate is 
			when IDLE =>						
				if CPU_sync ='1' then 
					nstate <= EXECUTING;
				else
					nstate <= IDLE;
			  	end if;
			when EXECUTING =>
				if CPU_interrupt ='1' then
					nstate <= IDLE;
				elsif instr_complete ='1' then
						nstate <= WAITING;
				else
					nstate <= EXECUTING;
				end if;
			
			when WAITING =>	 
				if CPU_interrupt ='1' then
					nstate <= IDLE;
				else
					nstate <= WAITING;
				end if;
			end case;
	end process NSTATE_PROC;
	
	CSTATE_PROC : process(clk) is
	begin			   
		if(clk'event and clk='0') then
			cstate <= nstate;
		end if;
		
	end process CSTATE_PROC;
	
	SHIFT_PROC:  process (cstate,CPU_opcode) is
	begin			
		if CPU_opcode ='0' and cstate = IDLE then
			start <= '1';
		else
			start <= '0';
		end if;	 
	end process SHIFT_PROC;		 
	
	RUN_PROC : process(cstate, CPU_opcode) is	
	begin
		if CPU_opcode ='1' and cstate =EXECUTING then
			run <='1';
		else
			run <='0';
		end if;
	end process RUN_PROC;
	
	SHIFT_CONTROLLER: entity controller
	port map(
		clk => clk,	
		count => instruction(1 downto 0),
    	start => start,
		shift => shift,
		complete => shift_complete
	);	   
	
	FINISH_PROC : process(cstate) is
	begin
		if cstate = WAITING then
			controller_waiting <= '1';
		else
			controller_waiting <= '0';
		end if;
			
	end process FINISH_PROC;   
	
	
	
	
	
end serial_controller;
