-------------------------------------------------------------------------------
--
-- Title       : shift-controller
-- Design      : sw_backend
-- Author      : Vegard
-- Company     : NTNU
-- Description : Generates a shift signal for the number of cycles specified by
-- 				 count
-------------------------------------------------------------------------------

library IEEE;
use IEEE.STD_LOGIC_1164.all;
use IEEE.std_logic_unsigned.all;

entity controller is
	generic(COUNT_SIZE : natural :=2);
	port(
		clk			: in  std_logic;
		start		: in  std_logic;
		count		: in  std_logic_vector(COUNT_SIZE-1 downto 0);
		shift		: out std_logic;
		complete	: out std_logic
		);
end controller;





architecture controller of controller is
	--number of shifts that is to be performed % 32 -1
	-- 000 = 1 shift
	-- 111 = 8 shifts etc..
	signal cnt 	: std_logic_vector(4 downto 0);
 	type state	is (IDLE,COUNTING);
	signal cstate, nstate : state;
	
begin
	
	NSTATE_PROC : process(cstate,start,cnt) is
	begin
		case cstate is
			when IDLE =>  
			
			complete <='0';
			
			if(start)='1' then
				nstate <= COUNTING;
			else
				nstate <= IDLE;
			end if;
			when COUNTING =>
				if(cnt = count) then
					nstate <= IDLE;
		    		complete <='1';
				else			  
					complete <='0';
					nstate <= COUNTING;
				end if;
			end case;	
	end process NSTATE_PROC;
	
	CSTATE_PROC : process(clk)is
	begin					   
		if clk'event and clk ='0' then
			cstate <= nstate;
		end if;
	end process CSTATE_PROC;
	
	SHIFT_PROC : process(cstate) is
	begin
		if cstate = IDLE then
			shift <='0';
		else
			shift <='1';	
		end if;
	end process SHIFT_PROC;
	
	COUNT_PROC : process(clk) is
	begin
		if(clk'event and clk ='0') then
			if cstate = IDLE then
				cnt <=(others => '0');
			else
				cnt <= cnt+1;
			end if;
		end if;
	end process COUNT_PROC;
	

end controller;













