-------------------------------------------------------------------------------
--
-- Title       : user_logic
-- Design      : sw_backend
-- Author      : Vegard
-- Company     : NTNU
-- Description : Moified platform studio template, top level backend entity.
-------------------------------------------------------------------------------

-- DO NOT EDIT BELOW THIS LINE --------------------
library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_unsigned.all;




entity user_logic is
  generic(
    -- ADD USER GENERICS BELOW THIS LINE ---------------
    --USER generics added here
    -- ADD USER GENERICS ABOVE THIS LINE ---------------

    -- DO NOT EDIT BELOW THIS LINE ---------------------
    -- Bus protocol parameters, do not add to or delete
    C_SLV_DWIDTH                   : integer              := 32;
    C_NUM_REG                      : integer              := 3
    -- DO NOT EDIT ABOVE THIS LINE ---------------------
  );
  port
  (
    -- ADD USER PORTS BELOW THIS LINE ------------------
    --USER ports added here
    -- ADD USER PORTS ABOVE THIS LINE ------------------

    -- DO NOT EDIT BELOW THIS LINE ---------------------
    -- Bus protocol ports, do not add to or delete
    Bus2IP_Clk                     : in  std_logic;
    Bus2IP_Reset                   : in  std_logic;
    Bus2IP_Data                    : in  std_logic_vector(0 to C_SLV_DWIDTH-1);
    Bus2IP_BE                      : in  std_logic_vector(0 to C_SLV_DWIDTH/8-1);
    Bus2IP_RdCE                    : in  std_logic_vector(0 to C_NUM_REG-1);
    Bus2IP_WrCE                    : in  std_logic_vector(0 to C_NUM_REG-1);
    IP2Bus_Data                    : out std_logic_vector(0 to C_SLV_DWIDTH-1);
    IP2Bus_RdAck                   : out std_logic;
    IP2Bus_WrAck                   : out std_logic;
    IP2Bus_Error                   : out std_logic
    -- DO NOT EDIT ABOVE THIS LINE ---------------------
  );

  attribute SIGIS : string;
  attribute SIGIS of Bus2IP_Clk    : signal is "CLK";
  attribute SIGIS of Bus2IP_Reset  : signal is "RST";

end entity user_logic;

------------------------------------------------------------------------------
-- Architecture section
------------------------------------------------------------------------------

architecture IMP of user_logic is

  --USER signal declarations added here, as needed for user logic

	constant CPU_REG_SIZE : natural :=32;
	constant SHIFT_SIZE : natural :=8;

	signal run			: std_logic;
	signal completion	: std_logic;
	signal shift				: std_logic;
	signal shift_o, shift_i 	: std_logic_vector(SHIFT_SIZE-1 downto 0);


	
	signal write			:  std_logic;	
	signal instruction8 	:  std_logic_vector(7 downto 0);
	


	--------------------------------------------
	-- Controller signals

	signal controller_waiting : std_logic;
	signal zerofill : std_logic;
	signal address  : std_logic_vector(1 downto 0);
	signal instruction : std_logic_vector(7 downto 0);

	-----------------------------------------------

	signal zero_gap : std_logic_vector(22 downto 0);
	signal sig_zeros : std_logic_vector(CPU_REG_SIZE-1 downto 0);

  ------------------------------------------
  -- Signals for user logic slave model s/w accessible register example
  ------------------------------------------
  signal slv_reg0                       : std_logic_vector(0 to C_SLV_DWIDTH-1);
  signal slv_reg1                       : std_logic_vector(0 to C_SLV_DWIDTH-1);
  signal slv_reg2                       : std_logic_vector(0 to C_SLV_DWIDTH-1);
  signal slv_reg_write_sel              : std_logic_vector(0 to 2);
  signal slv_reg_read_sel               : std_logic_vector(0 to 2);
  signal slv_ip2bus_data                : std_logic_vector(0 to C_SLV_DWIDTH-1);
  signal slv_read_ack                   : std_logic;
  signal slv_write_ack                  : std_logic;


	

begin

  --USER logic implementation added here

  ------------------------------------------
  -- Example code to read/write user logic slave model s/w accessible registers
  -- 
  -- Note:
  -- The example code presented here is to show you one way of reading/writing
  -- software accessible registers implemented in the user logic slave model.
  -- Each bit of the Bus2IP_WrCE/Bus2IP_RdCE signals is configured to correspond
  -- to one software accessible register by the top level template. For example,
  -- if you have four 32 bit software accessible registers in the user logic,
  -- you are basically operating on the following memory mapped registers:
  -- 
  --    Bus2IP_WrCE/Bus2IP_RdCE   Memory Mapped Register
  --                     "1000"   C_BASEADDR + 0x0
  --                     "0100"   C_BASEADDR + 0x4
  --                     "0010"   C_BASEADDR + 0x8
  --                     "0001"   C_BASEADDR + 0xC
  -- 
  ------------------------------------------
  slv_reg_write_sel <= Bus2IP_WrCE(0 to 2);
  slv_reg_read_sel  <= Bus2IP_RdCE(0 to 2);
  slv_write_ack     <= Bus2IP_WrCE(0) or Bus2IP_WrCE(1) or Bus2IP_WrCE(2);
  slv_read_ack      <= Bus2IP_RdCE(0) or Bus2IP_RdCE(1) or Bus2IP_RdCE(2);

  -- implement slave model software accessible register(s)
  SLAVE_REG_WRITE_PROC : process( Bus2IP_Clk ) is
  begin

    if Bus2IP_Clk'event and Bus2IP_Clk = '1' then
      if Bus2IP_Reset = '1' then
        slv_reg0(0 to 15) <= (others => '0');
      else
        case slv_reg_write_sel is
          when "100" =>
            for byte_index in 0 to 1 loop
              if ( Bus2IP_BE(byte_index) = '1' ) then
                slv_reg0(byte_index*8 to byte_index*8+7) <= Bus2IP_Data(byte_index*8 to byte_index*8+7);
              end if;
            end loop; 
          when others => null;
        end case;
      end if;
    end if;

  end process SLAVE_REG_WRITE_PROC;

  INSTR_REG_PROC : process(Bus2IP_Clk) is
  begin
		if Bus2IP_Clk'event and Bus2IP_Clk = '1' then
			if Bus2IP_Reset = '1' then
			  slv_reg0(16 to 31) <= (others => '0');
			else
			  slv_reg0(31) <= controller_waiting;
			  slv_reg0(16 to 30) <= (others =>'0');
	    end if;
    end if;

  end process INSTR_REG_PROC;
	
  DR1_PROC : process( Bus2IP_Clk ) is
  begin
		if Bus2IP_Clk'event and Bus2IP_Clk = '1' then
			if slv_reg_write_sel="010" then
				write <='1';
			else
				write <='0';
			end if;
		end if;
   end process DR1_PROC;

  -- implement slave model software accessible register(s) read mux
  SLAVE_REG_READ_PROC : process( slv_reg_read_sel, slv_reg0, slv_reg1, slv_reg2 ) is
  begin

    case slv_reg_read_sel is
      when "100" => slv_ip2bus_data <= slv_reg0;
      when "010" => slv_ip2bus_data <= slv_reg1;
      when "001" => slv_ip2bus_data <= slv_reg2;
      when others => slv_ip2bus_data <= (others => '0');
    end case;

  end process SLAVE_REG_READ_PROC;

  ------------------------------------------
  -- Example code to drive IP to Bus signals
  ------------------------------------------
  IP2Bus_Data  <= slv_ip2bus_data when slv_read_ack = '1' else
                  (others => '0');

  IP2Bus_WrAck <= slv_write_ack;
  IP2Bus_RdAck <= slv_read_ack;
  IP2Bus_Error <= '0';


	zero_gap <= (others =>'0');
	sig_zeros <= (others =>'0');
	
	DATA_IN : entity register_macro  	  
	generic map(
		C_SIZE => SHIFT_SIZE, 
		C_NUM_REG =>CPU_REG_SIZE/SHIFT_SIZE
		)
	port map(
		clk			=> Bus2IP_Clk,
		data_i 		=> Bus2IP_Data,
		shift_i		=> sig_zeros(7 downto 0),
		write	 	=> write,
		shift 		=> shift,
		data_o		=> slv_reg1,
		shift_o		=> shift_i
	
	);	   	
	
	DATA_OUT : entity register_macro
	generic map(
		C_SIZE => SHIFT_SIZE, 
		C_NUM_REG =>CPU_REG_SIZE/SHIFT_SIZE
		)
	port map(
	   	clk 	=> Bus2IP_Clk, 
		data_i  => sig_zeros,
		shift_i => shift_o,
		write   => sig_zeros(0),
		shift 	=> shift,
		shift_o => open,
		data_o => slv_reg2
	);
	
	SERIAL_CONTROLLER : entity serial_controller
	port map(
		clk 			=> Bus2IP_Clk,	 
		instruction		=> slv_reg0(0 to 7),
		HW_complete 	=> completion,
		shift			=> shift,
		controller_waiting => controller_waiting,
		run 			=> run,
		zerofill		=> zerofill,
		address			=> address
		);
		
---------------------------------------------------------
-- Sequential multiplicator test

	MULT : entity multiplier
	generic map(
		OPERAND_SIZE => 32,
		LOG_OPERAND_SIZE => 5
	)
	port map(
		clk => Bus2IP_Clk,		
		reset => Bus2IP_Reset,
		addr => address(0),
		zerofill => zerofill,
		shift_i => shift_i,
		shift_o => shift_o,
		shift_g => shift,
		run 	=> run,
		finish  => completion			
	);











end IMP;
