-------------------------------------------------------------------------------
--
-- Title       : register_di
-- Design      : addressable_registers
-- Author      : Vegard Endresen
-- Company     : NTNU
-- Description : Dual input register
-------------------------------------------------------------------------------

library IEEE;
use IEEE.STD_LOGIC_1164.all;

entity register_di is
   generic(
		C_SIZE : natural := 8
	);						  
	port(
	  clk 	  : in std_logic;
	  reset	  : in std_logic;
	  write0  : in std_logic; -- control input0
	  write1  : in std_logic; -- control input1
	  data_i0 : in std_logic_vector(C_SIZE-1 downto 0); -- input0
	  data_i1 : in std_logic_vector(C_SIZE-1 downto 0);	-- input1
	  data_o  : out std_logic_vector(C_SIZE-1 downto 0)
	);
end register_di;


architecture RTL of register_di is

signal memory : std_logic_vector(C_SIZE-1 downto 0);

begin 
	
	data_o <= memory;
	
	MEMORY_PROC : process(clk) is
	begin
		if clk'event and clk='1' then 
			if reset ='1' then
				memory <=(others =>'0');
			elsif write0 ='1' then 
				memory <= data_i0;
			elsif write1='1' then
				memory <= data_i1;
			else
				memory <= memory;
			end if;	   
		end if;
	end process MEMORY_PROC;

end RTL;
