-------------------------------------------------------------------------------
--
-- Title       : register_di_shift
-- Design      : addressable_registers
-- Author      : Vegard Endresen
-- Company     : NTNU
-- Description : Dual input register with serial shift capability
-------------------------------------------------------------------------------

library IEEE;
use IEEE.STD_LOGIC_1164.all;

entity register_di_shift is
 generic(
		C_SIZE : natural := 8
	);						  
	port(
	  clk 	  : in std_logic;
	  reset	  : in std_logic;
	  shift	  : in std_logic;
	  write0  : in std_logic;
	  write1  : in std_logic;
	  data_i0 : in std_logic_vector(C_SIZE-1 downto 0);
	  data_i1 : in std_logic_vector(C_SIZE-1 downto 0);
	  data_o  : out std_logic_vector(C_SIZE-1 downto 0)
	);
end register_di_shift;


architecture RTL of register_di_shift is

signal memory : std_logic_vector(C_SIZE-1 downto 0);

begin
	data_o <= memory;
	
	MEMORY_PROC : process(clk) is
	begin
		if clk'event and clk='1' then 
			if reset ='1' then
				memory <=(others =>'0');
			elsif shift ='1' then
				memory(C_SIZE-1) <='0';
				for i in C_SIZE-2 downto 0 loop
					memory(i+1) <= memory(i);
				end loop;
			elsif write0 ='1' then 
				memory <= data_i0;
				
			elsif write1 <='1' then
				memory <= data_i1;
			else
				memory <= memory;
			end if;	   
		end if;
	end process MEMORY_PROC;
	

end RTL;
