-------------------------------------------------------------------------------
--
-- Title       : testbench
-- Design      : paralell_registers
-- Author      : Vegard	Endresen
-- Company     : NTNU
-- Description : Testbench for parallel register
-------------------------------------------------------------------------------

library IEEE;
use IEEE.STD_LOGIC_1164.all;

entity testbench is
end testbench;

architecture TEST of testbench is


-- Common field although not all fields are used by all components

constant C_SIZE 	: natural :=8;
constant C_NUM_REG	: natural :=4;
constant C_GSIZE 	: natural :=4;
constant T_CLK  	: time :=10 ns;

signal clk 		: std_logic; 
signal reset	: std_logic;
signal shift	: std_logic; 
signal write	: std_logic;
signal shift_i	: std_logic_vector(C_SIZE-1 downto 0);
signal shift_o	: std_logic_vector(C_SIZE-1 downto 0);
signal dataI	: std_logic_vector(C_SIZE*C_NUM_REG-1 downto 0);
signal dataO 	: std_logic_vector(C_SIZE*C_NUM_REG-1 downto 0);

------------------------------------------------------------
-- register_macro_shift fields

constant C_INPUT_SIZE : natural :=8;

signal shift_p : std_logic;
signal shift_s : std_logic;
------------------------------------------------------------
begin
	
	
  -- Clock generation
  CLKGEN: process is
  begin
    clk <= '1';
    wait for T_CLK/2;
    clk <= '0';
    wait for T_CLK/2;
  end process;
  
------------------------------------------------------------
---- Register macro test
-- 
--  REGISTER_MACRO : entity register_macro
--  generic map(
--  	C_SIZE => C_SIZE,
--    	C_NUM_REG => C_NUM_REG
--  )
--  port map(
--  	clk => clk,
--		reset => reset,
--  	write => write,
--		data_i => dataI,
--		data_o => dataO,
--		shift  => shift,
--		shift_i => shift_i,
--		shift_o => shift_o
--  );	
--  
--  STIMULI : process is
--  begin
--	  dataI <=x"ABCDEF12";
--	  reset <='0';
--	  write <='0';
--	  shift <='0';
--	  wait for T_CLK/2;
--	  
--	  write <='1'; 
--	  
--	  wait for T_CLK;
--	  
--	  write <='0';
--	  shift <='1';
--	  
--	  for i in 0 to C_NUM_REG-1 loop
--		  shift_i <= dataI(C_SIZE*(i+1)-1 downto C_SIZE*i);
--		  wait for T_CLK;
--	  end loop;
--	  shift <='0';
--	  wait;	  
--  end process STIMULI;
------------------------------------------------------------ 

		


--------------------------------------------------------
-- register_macro_shift test
 
  REGISTER_MACRO : entity register_macro_s
  generic map(
  	C_SIZE => C_SIZE,
  	C_INPUT_SIZE => C_SIZE*C_NUM_REG,
    C_NUM_REG => C_NUM_REG
  )
  port map(
  	clk => clk,
  	reset => reset,
	write => write,
	data_i => dataI,
	data_o => dataO,
	shift_s  => shift_s,
	shift_p	=> shift_p,
	shift_i => shift_i,
	shift_o => shift_o
  );	
  
  STIMULI : process is
  begin
	  dataI <=x"ABCDEF12"; 
	  reset <='0';
	  write <='0';
	  shift <='0';
	  wait for T_CLK/2;
	  
	  write <='1'; 
	  
	  wait for T_CLK;
	  
	  write <='0';
	  shift <='1';
	  
	  for i in 0 to C_NUM_REG-1 loop
		  shift_i <= dataI(C_SIZE*(i+1)-1 downto C_SIZE*i);
		  wait for T_CLK;
	  end loop;
	  shift <='0';
	  wait;	  
  end process STIMULI;
-------------------------------------------------------- 



  
end TEST;
