-------------------------------------------------------------------------------
--
-- Title       : shift_register
-- Design      : serial_registers
-- Author      : Vegard	Endresen
-- Description : Serial shift register with full paralell and serial interface
-------------------------------------------------------------------------------

library IEEE;
use IEEE.STD_LOGIC_1164.all;

entity shift_register is
	generic(
		C_SIZE	: natural :=8 	--register size
	);

	port(
		clk			: in  std_logic;  
		reset		: in  std_logic;
		data_i   	: in  std_logic_vector(C_SIZE-1 downto 0);
		write		: in  std_logic;
		shift		: in  std_logic;
		shift_i		: in  std_logic;
		shift_o		: out std_logic;
		data_o		: out std_logic_vector(C_SIZE-1 downto 0)
	 );
	

end entity shift_register;
					  
architecture RTL of shift_register is 

signal memory 	: std_logic_vector(C_SIZE-1 downto 0);

begin
	
	data_o 	<= memory;
	shift_o <= memory(0);
	
	WRITE_PROC : process(clk) is
	begin
	if(clk'event and clk ='1') then
		if reset ='1' then
			memory <= (others =>'0');
		elsif shift ='1' then
			for i in C_SIZE-2 downto 0 loop
				memory(i) <= memory(i+1);
			end loop;
			memory(C_SIZE-1) <= shift_i;
		elsif write ='1' then
			memory <= data_i;
		else	
			memory <= memory;	
		end if;
	end if;
	end process WRITE_PROC; 
								
end architecture RTL;
