-------------------------------------------------------------------------------
--
-- Title       : addressable
-- Design      : generic_architectures
-- Author      : Vegard Endresen
-- Company     : NTNU
-- Description : Generic addressable architecture for auto-synthesis
-------------------------------------------------------------------------------

library IEEE;
use IEEE.STD_LOGIC_1164.all;

entity addressable is
generic(
	constant C_REG_SIZE : natural := 8;
	constant C_NUM_REG : natural :=8;
	constant C_ADDR_BITS : natural :=3
);
	port(
		clk			: in  std_logic;
		run			: in  std_logic;
		addr		: in  std_logic_vector(C_ADDR_BITS-1 downto 0);
		data_i		: in  std_logic_vector(C_REG_SIZE-1 downto 0);
		data_o		: out std_logic_vector(C_REG_SIZE-1 downto 0);
		complete	: out std_logic
	);
	

end addressable;


architecture RTL of addressable is

signal local_i : std_logic_vector(C_REG_SIZE*C_NUM_REG-1 downto 0);
signal sel : std_logic_vector(C_NUM_REG -1 downto 0);
signal write : std_logic_vector(C_NUM_REG-1 downto 0);

begin
	
	complete <='0';
	
	DECODER : entity decoder
		generic map(
			C_INPUT_SIZE => C_ADDR_BITS,
			C_OUTPUT_SIZE => C_NUM_REG
		)
		port map(
			input => addr,
			output => sel
		);
		
	MUX : entity multiplexer 
		generic map(
			C_NOI => C_NUM_REG,
			C_SIG_SIZE => C_REG_SIZE,
			C_ADDR_SIZE => C_ADDR_BITS
		)
		port map( 
			input => local_i,
			addr  => addr,
			output => data_o
		);
	REG_GEN: for i in 0 to C_NUM_REG-1 generate
		DATA_OUT : entity di_register
		generic map(
			C_SIZE => C_REG_SIZE
		)
		port map(
		    clk 	=> clk, 
			write 	=> write(i),
			sel 	=> sel(i),
			data_i0	=>  data_i,
			data_i1 => 	local_i((i+1)*C_REG_SIZE-1 downto (i)*C_REG_SIZE),
			data_o	 =>  local_i((i+1)*C_REG_SIZE-1 downto (i)*C_REG_SIZE)	
		);
	end generate;
	
	LOCAL_PROC : process(run) is
	begin
		if run ='1' then 
			write <= (others => '1');
		else
			write <= (others => '0');
		end if;
	
	end process LOCAL_PROC;
	
	
	
end RTL;
