-------------------------------------------------------------------------------
--
-- Title       : addressable_shift
-- Design      : generic architectures
-- Author      : Vegard
-- Company     : NTNU
-- Description : Generic hybrid architecture for auto-synthesis
-------------------------------------------------------------------------------

library IEEE;
use IEEE.STD_LOGIC_1164.all;

entity addressable_shift is
generic(
	constant C_REG_SIZE : natural := 8;
	constant C_NUM_REG : natural := 4;
	constant C_NUM_CHAINS : natural := 3;
	constant C_ADDR_BITS : natural :=2
);
	port(
		clk			: in  std_logic;
		reset		: in  std_logic;
		run			: in  std_logic;
		shift		: in  std_logic;
		addr		: in  std_logic_vector(C_ADDR_BITS-1 downto 0);
		shift_i		: in  std_logic_vector(C_REG_SIZE-1 downto 0);
		shift_o		: out std_logic_vector(C_REG_SIZE-1 downto 0);
		complete	: out std_logic
	);
	

end addressable_shift;


architecture addressable_shift of addressable_shift is

signal local_i	: std_logic_vector(C_REG_SIZE*C_NUM_REG*C_NUM_CHAINS-1 downto 0);
signal shift_il	: std_logic_vector(C_NUM_REG*C_NUM_CHAINS downto 0);
signal shift_ol	:std_logic_vector(C_NUM_REG*C_NUM_CHAINS downto 0);

signal write  :std_logic_vector(C_NUM_REG*C_NUM_CHAINS-1 downto 0);


signal sel : std_logic_vector(2 downto 0);

begin
	
	complete <='0';
	
	SEL_DECODER : entity decoder
		generic map(
			C_INPUT_SIZE => C_ADDR_BITS,
			C_OUTPUT_SIZE => C_NUM_CHAINS
		)
		port map(
			input => addr,
			output => sel
	);
		
	
	MUX : entity multiplexer 
	generic map(
		C_NOI => C_NUM_CHAINS,
		C_SIG_SIZE => C_REG_SIZE,
		C_ADDR_SIZE => C_ADDR_BITS
	)
	port map( 
		input => local_i,
		addr  => addr,
		output => shift_o
	);
	
	REG_GEN0: for i in 0 to C_NUM_CHAINS-1 generate
		DATA_OUT : entity shift_chain
		generic map(
			C_SIZE => C_REG_SIZE,
			C_NUM_REG=> C_NUM_REG
		)
		port map(
		    clk 	=> clk, 
			reset   => reset,
			sel     => sel(i),
			data_i	=>  local_i((i+1)*C_REG_SIZE*C_NUM_REG-1 downto (i)*C_REG_SIZE*C_NUM_REG),
			write 	=> write((i+1)*C_NUM_REG-1 downto i*C_NUM_REG),
			shift 	=> shift,
			shift_i => shift_il((i+1)*C_REG_SIZE-1 downto (i)*C_REG_SIZE),	
			shift_o => shift_ol((i+1)*C_REG_SIZE-1 downto (i)*C_REG_SIZE),
			data_o	 =>  local_i((i+1)*C_REG_SIZE*C_NUM_REG-1 downto (i)*C_REG_SIZE*C_NUM_REG)	
		);
	end generate;	

	
end addressable_shift;
