-------------------------------------------------------------------------------
--
-- Title       : register_macro
-- Design      : parallel_registers
-- Author      : Vegard	Endresen
-- Company     : NTNU
-- Description : Composed of chained parallel registers creating a  
--				 parallel shift register macro.
-------------------------------------------------------------------------------

library IEEE;
use IEEE.STD_LOGIC_1164.all;

entity register_macro is 
	generic(											  
		C_SIZE	 	: natural := 8; -- size of each register
		C_NUM_REG 	: natural := 4	-- number of registers
	);
	port(	   
		clk		: in  std_logic;
		reset	: in  std_logic;
		shift	: in  std_logic;
		write	: in  std_logic;
		data_i	: in  std_logic_vector(C_SIZE*C_NUM_REG -1 downto 0); 
		shift_i : in  std_logic_vector(C_SIZE-1 downto 0);
		data_o	: out std_logic_vector(C_SIZE*C_NUM_REG -1 downto 0);
		shift_o	: out std_logic_vector(C_SIZE-1 downto 0)
	);
	
end register_macro;

architecture RTL of register_macro is

signal reg_inputs 	: std_logic_vector(C_SIZE*C_NUM_REG -1 downto 0);  
signal reg_outputs 	: std_logic_vector(C_SIZE*C_NUM_REG -1 downto 0);  
signal write_i : std_logic;

begin 		 
	
	write_i <= write or shift;
	
	OUT_PROC : process(reg_outputs)
	begin				
		shift_o <= reg_outputs(C_SIZE-1 downto 0);
		data_o <= reg_outputs;	
	end process OUT_PROC;
	
	MUX_PROC : process(shift,shift_i,write,data_i,reg_outputs) is
	begin
		-- shift operation selected
		if shift ='1' then 
			reg_inputs(C_SIZE*C_NUM_REG -1 downto C_SIZE*(C_NUM_REG-1)) <= shift_i;
			for i in C_NUM_REG-1 downto 1 loop
				reg_inputs(C_SIZE*i -1 downto C_SIZE*(i-1)) <= reg_outputs(C_SIZE*(i+1) -1 downto C_SIZE*(i));
			end loop;
		-- parallel write operation selceted
		else
			for i in C_NUM_REG downto 1 loop
				reg_inputs(C_SIZE*i -1 downto C_SIZE*(i-1)) <= data_i(C_SIZE*i -1 downto C_SIZE*(i-1));
			end loop;  
		end if;
	end process MUX_PROC;
	
	REG_GEN: for i in C_NUM_REG downto 1 generate
		REG : entity parallel_register
			port map(
				clk => clk,
				reset => reset,
				data_i => reg_inputs(i*8-1 downto (i-1)*8),
				write => write_i,
				data_o => reg_outputs(i*8-1 downto (i-1)*8)	
				);
	end generate;

end RTL;
