-------------------------------------------------------------------------------
--
-- Title       : bram_id_cache
-- Design      : cpu_interface
-- Author      : Vegard
-- Company     : NTNU
-- Description : 
-------------------------------------------------------------------------------


library IEEE;
use IEEE.STD_LOGIC_1164.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_unsigned.all;

entity bram_id_cache is
 

end bram_id_cache;


architecture bram_id_cache of bram_id_cache is
 
 ----------------------------------------------------------------------------
 -- Perhipheral interface signals 
 
 ----------------------------------------------------------------------------
 
  constant C_SLV_AWIDTH     : integer := 32;
  constant C_SLV_DWIDTH     : integer := 32;
  constant C_NUM_REG			 : integer  := 2;
  constant C_NUM_MEM        : integer := 1;
  constant INSTR_SIZE		: natural := 16; 
  constant INSTR_ADDR_BITS : natural :=5;
 constant T_CLK : time :=10 ns;

 signal	Bus2IP_Clk  :   std_logic;
 signal Bus2IP_Reset :  std_logic;
 signal Bus2IP_Addr  : 	std_logic_vector(0 to C_SLV_AWIDTH-1);
 signal Bus2IP_CS    :  std_logic_vector(0 to C_NUM_MEM-1);
 signal Bus2IP_RNW   :  std_logic;
 signal Bus2IP_Data  :  std_logic_vector(0 to C_SLV_DWIDTH-1);
 signal Bus2IP_BE    : std_logic_vector(0 to C_SLV_DWIDTH/8-1);
 signal  Bus2IP_RdCE     :  std_logic_vector(0 to C_NUM_REG-1);
 signal  Bus2IP_WrCE      :   std_logic_vector(0 to C_NUM_REG-1);
 signal IP2Bus_Data  :  std_logic_vector(0 to C_SLV_DWIDTH-1);
 signal IP2Bus_RdAck :  std_logic;
signal  IP2Bus_WrAck :  std_logic;
 signal IP2Bus_Error :  std_logic;
 
 
  ---------------------------------------------------------------------
  -- HW - BRAM signals
signal bram_hw_en			:  std_logic;
signal bram_hw_we   		:  std_logic;
signal bram_hw_addr			:  std_logic_vector(7 downto 0);
signal bram_hw_data_i 		:  std_logic_vector(C_SLV_DWIDTH-1 downto 0);
signal bram_hw_data_o 		:  std_logic_vector(C_SLV_DWIDTH-1 downto 0);
signal bram_hw_read_ack 	: std_logic;
signal bram_hw_write_ack 	: std_logic;

---------------------------------------------------------------------		
  
 ------------------------------------------
  -- Signals for user logic slave model s/w accessible register example
  ------------------------------------------
  signal slv_reg0                       : std_logic_vector(0 to C_SLV_DWIDTH-1); -- sync/control
  signal slv_reg1                       : std_logic_vector(0 to C_SLV_DWIDTH-1); -- reg0
  signal slv_reg2                       : std_logic_vector(0 to C_SLV_DWIDTH-1); --reg1
  signal slv_reg_write_sel              : std_logic_vector(0 to C_NUM_REG-1);
  signal slv_reg_read_sel               : std_logic_vector(0 to C_NUM_REG-1);
  signal slv_ip2bus_data                : std_logic_vector(0 to C_SLV_DWIDTH-1);
  signal slv_read_ack                   : std_logic;
  signal slv_write_ack                  : std_logic;
  
  

  signal execute : std_logic;
  
  ---------------------------------------------------------------------
  -- Shift controller signals
 
  signal shift			: std_logic;
  ---------------------------------------------------------------------		
  
  
  ---------------------------------------------------------------------
  -- Instruction controller signals
  
  
  signal id_en : std_logic;
  signal id_we : std_logic;
  signal id_read_ack : std_logic;
  
	signal instruction 		: std_logic_vector(INSTR_SIZE-1 downto 0);
	signal id_addr		    : std_logic_vector(7 downto 0);
	signal register_write	: std_logic;

	signal shift_mask		: std_logic;
	signal zero_mask		: std_logic;	
	signal instr_complete   : std_logic;
  ---------------------------------------------------------------------
  
  ---------------------------------------------------------------------
  -- Instruction fetcher signals
  
    signal if_en		  	:  std_logic;
  	signal if_read_ack 		:  std_logic;	
	signal if_instr_write	:  std_logic;
	signal if_instr_shift	:  std_logic;
	signal if_addr	:  std_logic_vector(INSTR_ADDR_BITS-1 downto 0)	;
  ---------------------------------------------------------------------
  
  
  
  ---------------------------------------------------------------------
  -- REG0 signals
  
  signal reg0_shift_o : std_logic_vector(0 to 7);
  signal reg0_data_o : std_logic_vector(31 downto 0);
  ---------------------------------------------------------------------
  
  
  ---------------------------------------------------------------------
  -- Multiplier signals
  signal hw_complete : std_logic;
  signal hw_run : std_logic;  
  signal shift_i, shift_o :std_logic_vector(7 downto 0);
  ---------------------------------------------------------------------
  
  	
  
 
signal zeros : std_logic_vector(31 downto 0); 
  
signal halt : std_logic;
  

signal mem_ip2bus_data : std_logic_vector(0 to 31);
signal mem_read_ack : std_logic;
signal mem_write_ack : std_logic;
signal mem_error : std_logic;


signal next_instr : std_logic;
												
signal instr_depth : std_logic_vector(7 downto 0); 

signal if_stall : std_logic;

begin	 
	
	
  if_stall <= Bus2IP_CS(0) or Bus2IP_WrCE(0);	
	
  instr_depth <=slv_reg0(0 to 7);	
	
  slv_reg_write_sel <= Bus2IP_WrCE(0 to 1);
  slv_reg_read_sel  <= Bus2IP_RdCE(0 to 1);
  slv_write_ack     <= Bus2IP_WrCE(0) or Bus2IP_WrCE(1);
  slv_read_ack      <= Bus2IP_RdCE(0) or Bus2IP_RdCE(1);
  
  
  next_instr <= if_instr_write or if_instr_shift;
  
  -- implement slave model software accessible register(s)
  SLAVE_REG_WRITE_PROC : process( Bus2IP_Clk ) is
  begin

    if Bus2IP_Clk'event and Bus2IP_Clk = '1' then
      if Bus2IP_Reset = '1' then
        slv_reg0 <= (others => '0');
      else
         
		  slv_reg0(16 to 30) <= (others =>'0');
		  slv_reg0(31) <= halt;
		  
		  case slv_reg_write_sel is
          when "10" =>
				-- 16 bit for sw write, 16 bit for hw write 
			  for byte_index in 0 to 2 loop
              if ( Bus2IP_BE(byte_index) = '1' ) then
                slv_reg0(byte_index*8 to byte_index*8+7) <= Bus2IP_Data(byte_index*8 to byte_index*8+7);
              end if;
			
				end loop;
				-- other regs are sw read only
--	    when "010" =>
--            for byte_index in 0 to (C_SLV_DWIDTH/8)-1 loop
--              if ( Bus2IP_BE(byte_index) = '1' ) then
--                slv_reg1(byte_index*8 to byte_index*8+7) <= Bus2IP_Data(byte_index*8 to byte_index*8+7);
--              end if;
--            end loop;
--          when "001" =>
--            for byte_index in 0 to (C_SLV_DWIDTH/8)-1 loop
--              if ( Bus2IP_BE(byte_index) = '1' ) then
--                slv_reg2(byte_index*8 to byte_index*8+7) <= Bus2IP_Data(byte_index*8 to byte_index*8+7);
--              end if;
--            end loop;
          when others => null;
        end case;
      end if;
    end if;

  end process SLAVE_REG_WRITE_PROC;


  -- implement slave model software accessible register(s) read mux
  SLAVE_REG_READ_PROC : process( slv_reg_read_sel, slv_reg0, slv_reg1, slv_reg2 ) is
  begin

    case slv_reg_read_sel is
      when "10" => slv_ip2bus_data <= slv_reg0;
      when "01" => slv_ip2bus_data <= slv_reg0;
	  when others => slv_ip2bus_data <= (others => '0');
    end case;

  end process SLAVE_REG_READ_PROC;
  
  TIMER_PROC : process(Bus2IP_Clk) is
  begin
	  if Bus2IP_Clk'event and Bus2IP_Clk='1' then
		  if slv_reg0(0) ='1' then
			 slv_reg1 <= (others =>'0'); 
		  elsif slv_reg0(1) ='1' then
		  	 slv_reg1 <= slv_reg1+1;						
		  else
			  slv_reg1 <= slv_reg1;
		  end if;
	  end if;
  end process TIMER_PROC;
  
  
	--****************************************SW********************************************
	----------------------------------------------------------------------------------------
	
	-- Writing the content of REG1 to address 0d255 is used as virtual NOP. 
	-- BRAM address 0d255 is thus reserved and cannot be used

	zeros <= (others =>'0');
	
	
	BRAM_UNIT : entity bram_packed 
	port map(
      	hw_clk 		=> Bus2IP_Clk,
		hw_en		 => bram_hw_en,
		hw_we   	 => bram_hw_we,
		hw_addr	 	=> bram_hw_addr,
		hw_data_i  =>	bram_hw_data_i,
		hw_data_o =>	bram_hw_data_o,
		hw_read_ack => bram_hw_read_ack,
		hw_write_ack => bram_hw_write_ack,
		
		Bus2IP_Clk                     => Bus2IP_Clk,
		Bus2IP_Reset                   => Bus2IP_Reset,
		Bus2IP_Addr                    => Bus2IP_Addr,
		Bus2IP_CS                      => Bus2IP_CS,
		Bus2IP_RNW                     => Bus2IP_RNW,
		Bus2IP_Data                    => Bus2IP_Data,
		Bus2IP_BE                      => Bus2IP_BE,
      IP2Bus_Data                    => mem_ip2bus_data,
      IP2Bus_RdAck                   => mem_read_ack,
      IP2Bus_WrAck                   => mem_write_ack,
      IP2Bus_Error                   => mem_error
    );
	

	IDECODER : entity instr_controller
		generic map(
		 	INSTR_SIZE => INSTR_SIZE
		)
		port map(
			clk => Bus2IP_Clk,
			instruction => instruction,
			read_ack => id_read_ack,
			write_ack => bram_hw_write_ack,
			hw_complete => hw_complete,
			next_instr => next_instr,
			address => id_addr,
			shift => shift,
			cs => Bus2IP_CS,
			halt => halt,
			register_write => register_write,
			hw_en => id_en,
			hw_we => id_we,
			shift_mask => shift_mask,
			zero_mask => zero_mask,
			hw_run => hw_run,
			instr_complete => instr_complete
	);
	
	REG0 : entity register_macro
		generic map(
			C_SIZE 		=> 8,
			C_NUM_REG  => 4
		)
		port map(
			clk =>Bus2IP_Clk,
			shift => shift,
			write => register_write,
			shift_i => zeros(7 downto 0) ,
			data_i => bram_hw_data_o,
			data_o => REG0_data_o,  -- open
			shift_o	=> shift_i
		);
		
		
	REG1 : entity register_macro
		generic map(
			C_SIZE 		=> 8,
			C_NUM_REG  => 4
		)
		port map(
			clk =>Bus2IP_Clk,
			shift => shift,
			shift_i => shift_o,
			write   => zeros(0),
			shift_o => open,
			data_i => zeros,
			data_o => bram_hw_data_i
		);		
		
	INSTRUCTION_REGISTER : entity register_macro
		generic map(
			C_SIZE => 16,
			C_NUM_REG => 2
		) 
		port map(
		  	clk		 => Bus2IP_Clk,
			data_i	 => bram_hw_data_o, 
			shift_i  => zeros(15 downto 0),
			write	 => if_instr_write,
			shift	 => if_instr_shift,
			data_o	 => open,
			shift_o	 => instruction
		);
		
	INSTRUCTION_FETCHER : entity instr_fetcher
		generic map(
			INSTR_ADDR_BITS => INSTR_ADDR_BITS
		)
		port map(
			clk => Bus2IP_Clk,
			reset => Bus2IP_Reset,
			halt => halt,
			execute => execute,
			read_ack => if_read_ack, 
			instr_depth => instr_depth(4 downto 0),
			if_en => if_en,
			instr_write => if_instr_write,
			instr_shift => if_instr_shift,
			instr_complete => instr_complete,
			instr_addr_o	 => if_addr
		);
		
	
	MULT : entity comparator_module 
	generic map(
		OPERAND_SIZE => 32,		   
		SHIFT_SIZE =>8,
		LOG_OPERAND_SIZE => 5
	)
	port map(
		clk =>  Bus2IP_Clk ,		
		reset => Bus2IP_Reset,
		addr => id_addr(0),
		zerofill => zero_mask,
		shift_i => shift_i,
		shift_o => shift_o,
		shift_g => shift,
		run 	=> hw_run,
		finish  => hw_complete
	);


		
		

	
	-- Multiplexing instruction fetch and data operations on BRAM
	
	
	ADDRESS_PROC : process(if_en,id_addr,if_addr) is
	begin
		if if_en ='1' then
			bram_hw_addr(INSTR_ADDR_BITS-1 downto 0) <= if_addr;  
			bram_hw_addr(7 downto INSTR_ADDR_BITS) <= (others => '0');  
		else
			bram_hw_addr <= id_addr;
		end if;
	end process ADDRESS_PROC;
	
	ENABLE_PROC : process(if_en,id_en,id_we) is
	begin
		if if_en ='1' then
			bram_hw_en <= if_en;
			bram_hw_we <='0';
		else
			bram_hw_en <= id_en;
			bram_hw_we <= id_we;
		end if;
	end process ENABLE_PROC;
	
	
	ACK_PROC : process(if_en,bram_hw_read_ack) is
	begin					
		if_read_ack <= bram_hw_read_ack  and if_en;
		
	end process ACK_PROC;
	
	ACK_PROC1 : process(if_en,bram_hw_read_ack) is
	begin					
		id_read_ack <= bram_hw_read_ack and not if_en;
	end process ACK_PROC1;		
	
	
	
------------------------------------------
  -- Example code to drive IP to Bus signals
  ------------------------------------------
  IP2Bus_Data  <= slv_ip2bus_data when slv_read_ack = '1' else
                  mem_ip2bus_data when mem_read_ack = '1' else
                  (others => '0');

  IP2Bus_WrAck <= slv_write_ack or mem_write_ack;
  IP2Bus_RdAck <= slv_read_ack or mem_read_ack;
  IP2Bus_Error <= '0';
  
  
  
  
  
  
  CLKGEN : process is
  begin
  	Bus2IP_clk <='0';
  	wait for T_CLK/2;
	Bus2IP_clk <='1';
	wait for T_CLK/2;
  end process CLKGEN;
  
  
  STIMULI : process is 
	type ArrayType is array(31 downto 0) of std_logic_vector(31 downto 0); 
	variable RealAddress  : std_logic_vector(0 to 7);
	variable dataArray : ArrayType;
	begin				 
		
		-- MICROPROGRAM SIMULATION
		-- compute abcdef12^2 from interrupted state and store answer in BRAM[19] & BRAM[18]  
		-- compute 12345678^2 from skratch and store in  BRAM[21] & BRAM[20]  

		dataArray(0) :=x"8003000B";
		dataArray(1) :=x"8007000C"; 
		dataArray(2) :=x"8007C000";
		dataArray(3) :=x"0000400D";
		dataArray(4) :=x"F000F000";
		dataArray(5) :=x"0010F000";	 --<-<-<-<-<------------------------------------
		dataArray(6) :=x"40128013";
		dataArray(7) :=x"80130011";
		dataArray(8) :=x"F0004013";
		dataArray(9) :=x"40148013";
		dataArray(10) :=x"40158013";
		dataArray(11) :=x"ABCDEF12";
		dataArray(12) :=x"ABCDEF12"; 
		dataArray(13) :=x"ABCDEF12";
		dataArray(14) :=x"00ABCDEF";
		dataArray(15) :=x"0000004C";
		dataArray(16) :=x"12345678";
		dataArray(17) :=x"12345678";
		
		Bus2IP_RdCE     <= (others =>'0');
 		Bus2IP_WrCE     <= (others =>'0');
		Bus2IP_Reset <= '1'; 
		Bus2IP_RNW   <='0';
		Bus2IP_Addr  <=(others => '0');
		RealAddress := (others => '0');
		Bus2IP_CS    <= (others => '1');
		Bus2IP_Data  <=(others => '0');
		Bus2IP_BE <= (others => '1');	  
		wait for T_CLK;
		Bus2IP_Reset<='0';
		wait for T_CLK;	  
		Bus2IP_Data <=x"F000F000";
		wait for T_CLK;	 
		RealAddress := RealAddress +1;
		--write program and data to BRAM
		for	i in 1 to 20 loop
			if(i /=255) then 
				Bus2IP_Data  <= (others =>'0');
			end if;
			Bus2IP_Addr(C_SLV_AWIDTH-10 to C_SLV_AWIDTH-3) <= RealAddress; 
			Bus2IP_Data <= dataArray(i);
			wait for T_CLK;
			RealAddress := RealAddress +1;
		end loop;	
		Bus2IP_CS(0) <='0';	 
		
		wait for 10*T_CLK;	
		--write instr_depth	to slv_reg0
		Bus2IP_WRCE <="10";
		Bus2IP_Data <=x"8A000000";
		wait for T_CLK;	 
		Bus2IP_WRCE <="00"; 
		wait for 5*T_CLK;	
		Bus2IP_WRCE <="10";
		Bus2IP_Data <=x"4A000000";
		wait for T_CLK;	 
		Bus2IP_WRCE <="00";
		Bus2IP_CS(0) <='1';
		Bus2IP_Data <= dataArray(0)	;
		Bus2IP_Addr(C_SLV_AWIDTH-10 to C_SLV_AWIDTH-3) <= (others =>'0');
		wait for 4*T_CLK;	 
		Bus2IP_CS(0) <='0';
		
		while halt ='0' loop
			wait for T_CLK;
		end loop;
		Bus2IP_WRCE <="10";
		Bus2IP_Data <=x"0A000000";
		wait for T_CLK;	 
		Bus2IP_WRCE <="00";
		
		
		
	wait;
	end process STIMULI;	
	
	
  
	
end bram_id_cache;
