-------------------------------------------------------------------------------
--
-- Title       : testbench_module
-- Design      : comparator
-- Author      : Vegard
-- Company     : NTNU
--
-------------------------------------------------------------------------------
--
-- File        : testbench_module.vhd
-- Generated   : Tue May 25 15:30:56 2010
-- From        : interface description file
-- By          : Itf2Vhdl ver. 1.20
--
-------------------------------------------------------------------------------
--
-- Description : 
--
-------------------------------------------------------------------------------

--{{ Section below this comment is automatically maintained
--   and may be overwritten
--{entity {testbench_module} architecture {testbench_module}}

library IEEE;
use IEEE.STD_LOGIC_1164.all;

entity testbench_module is
end testbench_module;

--}} End of automatically maintained section

architecture testbench_module of testbench_module is

	constant T_CLK : time := 10ns;

	constant OPERAND_SIZE      	: natural	:=32;
	constant SHIFT_SIZE			: natural   :=8;
	constant LOG_OPERAND_SIZE	: natural   :=5;
	
	signal clk			: std_logic;
	signal reset		: std_logic;
	signal run			: std_logic; 
	signal addr			: std_logic;
	signal zerofill		: std_logic;
	signal shift_i 		: std_logic_vector(SHIFT_SIZE-1 downto 0);
	signal shift_g		: std_logic;
	signal shift_o 		: std_logic_vector(SHIFT_SIZE-1 downto 0);
	signal finish		: std_logic;
	
	signal sim_data_i,sim_data_o : std_logic_vector(OPERAND_SIZE-1 downto 0);
	
	signal reg0_wr : std_logic;
	signal zeros   : std_logic_vector(OPERAND_SIZE-1 downto 0);
	
begin
	
	zeros <= (others =>'0');
	
	CLK_GEN : process is
	begin
		clk <='0'; 
		wait for T_CLK/2;
		clk <='1';
		wait for T_CLK/2;
	end process CLK_GEN;
	
	COMP_MOD : entity comparator_module 
		generic map(
			OPERAND_SIZE => OPERAND_SIZE,
			SHIFT_SIZE => SHIFT_SIZE,
			LOG_OPERAND_SIZE => LOG_OPERAND_SIZE
		)
		port map(
			clk 	 => clk,		
			reset    => reset,
			addr 	 => addr,
			zerofill => zerofill,
			shift_i  => shift_i,
			shift_o  => shift_o,
			shift_g  => shift_g,
			run 	 => run,
			finish   => finish	
		);
		
	REG0 : entity register_macro
		generic map(
			C_SIZE 		=> 8,
			C_NUM_REG  => 4
		)
		port map(
			clk 	=> clk	,
			shift	=> shift_g,
			write 	=> reg0_wr,
			shift_i => zeros(7 downto 0) ,
			data_i  => sim_data_i,
			data_o  => open,  -- open
			shift_o	=> shift_i
		);
		
		
	REG1 : entity register_macro
		generic map(
			C_SIZE 		=> 8,
			C_NUM_REG  => 4
		)
		port map(
			clk =>clk,
			shift => shift_g,
			shift_i => shift_o,
			write   => zeros(0),
			shift_o => open,
			data_i => zeros,
			data_o => sim_data_o
		);			
		
		
		
	STIMULI : process is
	begin
		reset <='1';
		addr  <='0';
		zerofill <='0';
		reg0_wr <= '0';
		shift_g <= '0';
		run <='0';
		sim_data_i <= x"ABCDEF12";
		wait for T_CLK;			  
		reset <='0';
		reg0_wr <='1';
		wait for T_CLK;
		reg0_wr <='0';
		shift_g <='1';
		for i in 3 downto 0 loop
			wait for T_CLK;
		end loop;	  
		sim_data_i <= x"2BCDEF12";
		reg0_wr <='1'; 
		shift_g <='0';
		wait for T_CLK;
		reg0_wr <='0';
		shift_g <='1';
		addr <='1';
		for i in 3 downto 0 loop
			wait for T_CLK;
		end loop;
		shift_g <='0';
		run	<='1';
		while finish ='0' loop
			wait for T_CLK;
		end loop;  
		run <='0';	
		wait for T_CLK;
		shift_g <='1';
		for i in 3 downto 0 loop
			wait for T_CLK;
		end loop;	  
		shift_g <= '0';
		wait;
		
	end process STIMULI;
	

end testbench_module;
