-------------------------------------------------------------------------------
--
-- Title       : Backend
-- Design      : migration
-- Author      : Vegard
-- Company     : NTNU
-- File        : Backend.vhd
-- Description : 
-------------------------------------------------------------------------------


library IEEE;
use IEEE.STD_LOGIC_1164.all;

entity Backend is
end Backend;


architecture Backend of Backend is	

constant T_CLK : time :=10ns;


constant DATA_SIZE			: natural   :=32;
constant SHIFT_SIZE			: natural   :=8;	   
constant ADDR_SIZE			: natural   :=8;

--Global clock
signal clk		: std_logic;


-- Reconfigurable module signals
signal reset	: std_logic;
signal grant	: std_logic;
signal run		: std_logic; 
signal addr		: std_logic;
signal zerofill	: std_logic;
signal shift_i 	: std_logic_vector(SHIFT_SIZE-1 downto 0);
signal shift_g	: std_logic;	
signal request	: std_logic_vector(1 downto 0);
signal shift_o 	: std_logic_vector(SHIFT_SIZE-1 downto 0);
signal finish	: std_logic; 



-- RAM signals
signal clka		: std_logic;
signal dina		: std_logic_VECTOR(11 downto 0);
signal addra	: std_logic_VECTOR(7 downto 0);
signal wea		: std_logic_VECTOR(0 downto 0);
signal clkb		: std_logic;
signal addrb	: std_logic_VECTOR(7 downto 0);
signal ssrb		: std_logic;
signal doutb	: std_logic_VECTOR(11 downto 0);


-- Reconfigurable module signals

signal ram_rd,ram_rst : std_logic;
signal ram_wr : std_logic_vector(0 downto 0);
signal sig_zeros : std_logic_vector(DATA_SIZE-1 downto 0);

signal data_o : std_logic_vector(DATA_SIZE-1 downto 0);

signal dina_extended  : std_logic_vector(DATA_SIZE-1 downto 0);
signal doutb_extended : std_logic_vector(DATA_SIZE-1 downto 0);

signal duuuh : std_logic_vector(DATA_SIZE-1 downto 0);
signal ram_addr : std_logic_vector(ADDR_SIZE-1 downto 0);

begin
	
	dina_extended(11 downto 0) <= dina;
	doutb_extended(11 downto 0) <= doutb;
	
	dina_extended(DATA_SIZE-1 downto 12) <=  (others =>'0');
	doutb_extended(DATA_SIZE-1 downto 12) <=  (others =>'0');
	
	sig_zeros <= (others =>'0');
	
	CLK_PROC : process is
	begin
		clk <='0';
		wait for T_CLK/2;
		clk <='1';
		wait for T_CLK/2;		
	end process CLK_PROC;
	
	clka <= clk;
	clkb <= clk;
	
	RFM : entity  reconfigurable_module
		generic map(
			DATA_SIZE  => DATA_SIZE,
			SHIFT_SIZE => SHIFT_SIZE,
			ADDR_SIZE  => ADDR_SIZE
		)
		port map(
			clk			=> clk,
			reset		=> reset,
			grant		=> grant, 
			run			=> run,
			addr		=> addr,
			zerofill	=> zerofill,
			shift_g		=> shift_g,
			shift_i 	=> shift_i,
			shift_o 	=> shift_o,
			request		=> request,
			finish		=> finish
		);
		
	DATA_IN : entity register_macro  	  
	generic map(
		C_SIZE => SHIFT_SIZE, 
		C_NUM_REG =>DATA_SIZE/SHIFT_SIZE
		)
	port map(
		clk			=> clk,
		data_i 		=> doutb_extended,
		shift_i		=> sig_zeros(7 downto 0),
		write	 	=> sig_zeros(0),
		shift 		=> shift_g,
		data_o		=> doutb_extended,
		shift_o		=> shift_i
	
	);	   	
	
	DATA_OUT : entity register_macro
	generic map(
		C_SIZE => SHIFT_SIZE, 
		C_NUM_REG =>DATA_SIZE/SHIFT_SIZE
		)
	port map(
	   	clk 	=> clk, 
		data_i  => sig_zeros,
		shift_i => shift_o,
		write   => sig_zeros(0),
		shift 	=> shift_g,
		shift_o => open,
		data_o => duuuh
	);	
		
		
		
	RAM : entity ramModule
		port map(
		  clka => clka,
		  dina => duuuh(11 downto 0),
		  addra=> ram_addr,
		  wea => ram_wr,  
		  clkb => clkb,
		  addrb=> ram_addr,
		  ssrb => reset,
		  doutb => doutb
	);
	
	RAM_SLAVE : entity RAM_slave
		generic map(
			DATA_SIZE  => DATA_SIZE,
			SHIFT_SIZE => SHIFT_SIZE,
			ADDR_SIZE  => ADDR_SIZE
		)
		port map(
			clk		=> clk ,
			reset	=> reset,
			data_i  => duuuh,
			addr    => ram_addr,
			request => request, 
			shift_i => shift_o,
			shift_g	=> shift_g,
			grant	=> grant,
			ram_wr  => ram_wr,
			ram_rst => ram_rst
		);	
		
	STIMULI : process is
		begin
			reset <='1';
			wait for T_CLK;
			reset <='0';
			wait;
	end process STIMULI;	
		
		
	
end Backend;
