-------------------------------------------------------------------------------
--
-- Title       : bram_id_cache
-- Design      : cpu_interface
-- Author      : Vegard
-- Company     : NTNU
-- Description : RAM migration test
-------------------------------------------------------------------------------


library IEEE;
use IEEE.STD_LOGIC_1164.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_unsigned.all;

entity reconfigurable_module is
	generic(	 
		DATA_SIZE			: natural   :=32;
		SHIFT_SIZE			: natural   :=8;	   
		ADDR_SIZE			: natural   :=8
	);
	-- Extended reconfigurable interface
	port(
		clk			: in 	std_logic;
		reset		: in    std_logic;
		grant		: in 	std_logic;
		run			: in 	std_logic; 
		addr		: in    std_logic;
		zerofill	: in    std_logic;
		shift_i 	: in 	std_logic_vector(SHIFT_SIZE-1 downto 0);
		shift_g		: in 	std_logic;	
		request		: out	std_logic_vector(1 downto 0);
		shift_o 	: out 	std_logic_vector(SHIFT_SIZE-1 downto 0);
		finish		: out	std_logic 
	);

end reconfigurable_module;


architecture RTL of reconfigurable_module is


constant T_CLK : time :=10ns;

signal start 	: std_logic;
signal mode	 	: std_logic_vector(1 downto 0);   
signal data_i 	: std_logic_vector(DATA_SIZE-1 downto 0);
signal data_o	: std_logic_vector(DATA_SIZE-1 downto 0);
signal addr_i 	: std_logic_vector(ADDR_SIZE-1 downto 0);
signal complete : std_logic;

begin
				   
	
	RAM_MASTER : entity RAM_master
		generic map(
			DATA_SIZE  => DATA_SIZE,
			SHIFT_SIZE => SHIFT_SIZE,
			ADDR_SIZE  => ADDR_SIZE
		)
		port map(
			clk 		=> clk,				   
			start	 	=> start,
			shift		=> shift_g,
			grant	 	=> grant,
			mode  		=> mode,
			data_i 		=> data_i,
			shift_i 	=> shift_i,
			addr 		=> addr_i,
			request		=> request,
			data_o	 	=> data_o,
			shift_o 	=> shift_o,
			complete 	=> complete
		);				   
		
		
		STIMULI : process is
		begin
			
			start   <= '0';			
			mode    <= "10";
			data_i  <= x"ABCDEF12";
			addr_i  <= x"03";
			wait for T_CLK;
			while reset ='1' loop
				wait for T_CLK;
			end loop;		   
			
			start   <= '1';
			wait for T_CLK;
			start <='0';
			while complete ='0' loop
				wait for T_CLK;
			end loop;	
			wait for T_CLK;
			start  <='1';
			mode   <="01";
			addr_i <=x"03"; 
			wait for T_CLK;
			start <= '0';
			wait;
		end process STIMULI;
	
end RTL;
