-------------------------------------------------------------------------------
--
-- Title       : addr_counter
-- Design      : addressable_architecture
-- Author      : Vegard
-- Company     : NTNU
-- Description : Modulo counter that starts @ offset
-------------------------------------------------------------------------------

library IEEE;
use IEEE.STD_LOGIC_1164.all;
use IEEE.std_logic_unsigned.all;

entity addr_counter is 
	generic(C_ADDR_SIZE : natural :=5);
	port(
		clk			: in  std_logic;
		start		: in  std_logic;
		count		: in  std_logic_vector(C_ADDR_SIZE-1 downto 0);
		offset		: in  std_logic_vector(C_ADDR_SIZE-1 downto 0);
		address		: out std_logic_vector(C_ADDR_SIZE-1 downto 0);
		complete	: out std_logic
		);
end addr_counter;


architecture addr_counter of addr_counter is

	signal cnt 	: std_logic_vector(C_ADDR_SIZE-1 downto 0);
 	signal addr_cnt : std_logic_vector(C_ADDR_SIZE-1 downto 0);
	
	type state	is (IDLE,COUNTING);
	signal cstate, nstate : state;
	
begin
	
	address <= addr_cnt; 
	
	NSTATE_PROC : process(cstate,start,cnt,count) is
	begin
		case cstate is
			when IDLE =>  
			
			complete <='0';
			
			if(start)='1' then
				nstate <= COUNTING;
			else
				nstate <= IDLE;
			end if;
			when COUNTING =>
				if(cnt = count) then
					nstate <= IDLE;
		    		complete <='1';
				else			  
					complete <='0';
					nstate <= COUNTING;
				end if;
			end case;	
	end process NSTATE_PROC;
	
	CSTATE_PROC : process(clk)is
	begin					   
		if clk'event and clk ='0' then
			cstate <= nstate;
		end if;
	end process CSTATE_PROC;
	
	COUNT_PROC : process(clk) is
	begin
		if(clk'event and clk ='1') then
			if cstate = IDLE then
				cnt <=(others => '0');
				addr_cnt <= offset;
			else	  
				addr_cnt <= addr_cnt+1;
				cnt <= cnt+1;
			end if;
		end if;
	end process COUNT_PROC;
	

end addr_counter;
