-------------------------------------------------------------------------------
--
-- Title       : controller
-- Design      : addressable_architecture
-- Author      : Vegard
-- Company     : NTNU
-- Description : 
-------------------------------------------------------------------------------

library IEEE;
use IEEE.STD_LOGIC_1164.all;

entity controller is
	generic(
		C_ADDR_SIZE : natural :=5
	);
	port(
		signal clk 					: in  std_logic;
		signal CPU_interrupt		: in  std_logic; 
		signal CPU_write			: in  std_logic;
		signal CPU_read				: in  std_logic;
		signal CPU_run				: in  std_logic;
		signal processing_complete	: in  std_logic;
		signal count 				: in  std_logic_vector(C_ADDR_SIZE-1 downto 0);
		signal offset 				: in  std_logic_vector(C_ADDR_SIZE-1 downto 0);
		signal address 				: out std_logic_vector(C_ADDR_SIZE-1 downto 0);
		signal mode 				: out std_logic; 
		signal read					: out std_logic;
		signal run					: out std_logic;
		signal operation_complete	: out std_logic
	);
end controller;


architecture controller of controller is
type STATE is (IDLE,RW,RUNNING,WAITING);

signal cstate,nstate : STATE;
signal start 			: std_logic;
signal complete 		: std_logic;

signal readORwrite : std_logic ;

begin			   
	
	readORwrite <= CPU_read or CPU_write;
	
	ADDRESS_COUNTER : entity addr_counter 
	generic map(
		C_ADDR_SIZE => C_ADDR_SIZE
	)
	port map(
		clk		=> clk,
		start	=> start,
		count	=> count,
		offset	=> offset,
		address	=> address,
		complete => complete
	);
	
	NSTATE_PROC : process(cstate,CPU_interrupt,readORwrite,CPU_run,complete,processing_complete) is
	begin			   
		case cstate is 
			when IDLE =>						
				if readORwrite='1' then
						nstate <= RW;
				elsif CPU_run ='1' then 
						nstate <= RUNNING;
				else
					nstate <= IDLE;
			  	end if;
			when RW =>
				if CPU_interrupt ='1' then
					nstate <= IDLE;
				elsif complete ='1' then
					nstate <= WAITING;
				else
					nstate <= RW;
				end if;
			when RUNNING =>			 
				if CPU_interrupt ='1' then
					nstate <=IDLE;
				elsif processing_complete='1' then
					nstate <= WAITING;
				else
					nstate <= RUNNING;	  
				end if;
			when WAITING =>	 
				if CPU_interrupt ='1' then
					nstate <= IDLE;
				else
					nstate <= WAITING;
				end if;
			end case;
	end process NSTATE_PROC;
	
	CSTATE_PROC : process(clk) is
	begin			   
		if(clk'event and clk='0') then
			cstate <= nstate;
		end if;
		
	end process CSTATE_PROC;
	
	COUNT_PROC:  process (readORwrite,cstate) is
	begin			
		if readORwrite ='1' and cstate = IDLE then
			start <= '1';
		else
			start <= '0';
		end if;
	end process COUNT_PROC;
	
	RUN_PROC : process(cstate) is
	begin
		if cstate = RUNNING then 
			run <='1';
		else 
			run <='0';
		end if;
			
	end process RUN_PROC;
	
	--Toggle read or write mode
	MODE_PROC : process(cstate,CPU_read, CPU_write) is
	begin					   
		if cstate= RW and CPU_write='1' then
			mode <='1'; -- write
			read <='0';
		elsif cstate= RW and CPU_read='1' then
			mode <='0';	-- read
			read <='1';
		else
			mode <='0'; -- write
			read <='0';
		end if;
	end process MODE_PROC;
	
	OPC_PROC : process(cstate) is
	begin
		if cstate = WAITING then
			operation_complete <='1';
		else
			operation_complete <='0';
		end if;
		
	end process;
	
end controller;
