-------------------------------------------------------------------------------
--
-- Title       : di_gshift
-- Design      : addressable_architecture
-- Author      : Vegard
-- Company     : NTNU
-- Description : 
-------------------------------------------------------------------------------

library IEEE;
use IEEE.STD_LOGIC_1164.all;

entity di_gshift is	
	generic(
		C_SIZE : natural := 32;
		C_NUM_REGS : natural := 4;  
		C_INPUT_SIZE : natural :=5
	);
	port(
	  clk 	  : in std_logic;
	  shift	  : in std_logic;
	  write0  : in std_logic;
	  write1  : in std_logic_vector(C_NUM_REGS-1 downto 0);
	  data_i0 : in std_logic_vector(C_INPUT_SIZE-1 downto 0);
	  data_i1 : in std_logic_vector(C_SIZE/C_NUM_REGS-1 downto 0);
	  data_o  : out std_logic_vector(C_SIZE-1 downto 0)
	);
end di_gshift;


architecture di_gshift of di_gshift is

signal memory 	: std_logic_vector(C_SIZE-1 downto 0);
signal selected : std_logic;

begin
	data_o <= memory;
	
	selected <= write1(0) or write1(1) or write1(2) or write1(3)or write1(4) or write1(5) or write1(6) or write1(7);
	
	MEMORY_PROC : process(clk) is
	begin
		if clk'event and clk='1' then 
			if selected ='1' then 
				for i in C_NUM_REGS -1 downto 0 loop
					if write1(i) ='1' then
						memory((i+1)*C_SIZE/C_NUM_REGS-1 downto i*C_SIZE/C_NUM_REGS) <= data_i1;
					end if;
				end loop;
			elsif write0 ='1' then
				memory(C_SIZE-1 downto C_SIZE-C_INPUT_SIZE) <= data_i0;
				
			elsif shift ='1' then
				memory(C_SIZE-1) <= '0';
				for i in C_SIZE-2 downto 0 loop
					memory(i) <= memory(i+1);
				end loop;
			else
				memory <= memory;
			end if;	   
		end if;
	end process MEMORY_PROC;
	
end di_gshift;
