-------------------------------------------------------------------------------
--
-- Title       : register_macroPI
-- Design      : parallel_registers
-- Author      : Vegard
-- Company     : NTNU
-- Description : Register macro, parallel input (no shift input)
-------------------------------------------------------------------------------

library IEEE;
use IEEE.STD_LOGIC_1164.all;

entity register_macroPI is	
	generic(
		C_SIZE 		: natural := 8;	 -- size of each register
		C_NUM_REG 	: natural := 4	 -- number of registers
	);
	port(	   
		clk		: in  std_logic;
		data_i	: in  std_logic_vector(C_SIZE*C_NUM_REG -1 downto 0); 
		write	: in  std_logic;
		shift	: in  std_logic;
		data_o	: out std_logic_vector(C_SIZE*C_NUM_REG -1 downto 0);
		shift_o	: out std_logic_vector(C_SIZE-1 downto 0)
	);
	
	
	
end register_macroPI;


architecture RTL of register_macroPI is

	   
signal reg_inputs 	: std_logic_vector(C_SIZE*C_NUM_REG -1 downto 0);  
signal reg_outputs 	: std_logic_vector(C_SIZE*C_NUM_REG -1 downto 0);  
signal write_i : std_logic;

begin 		 
	
	write_i <= write or shift;
	
	OUT_PROC : process(reg_outputs)
	begin				
		shift_o <= reg_outputs(C_SIZE-1 downto 0);
		data_o <= reg_outputs;
		
	end process OUT_PROC;
	
	
	MUX_PROC : process(shift,write,data_i,reg_outputs) is
	begin
		if shift ='1' then 
			reg_inputs(C_SIZE*C_NUM_REG -1 downto C_SIZE*(C_NUM_REG-1)) <= (others => '0');
			for i in C_NUM_REG-1 downto 1 loop
				reg_inputs(C_SIZE*i -1 downto C_SIZE*(i-1)) <= reg_outputs(C_SIZE*(i+1) -1 downto C_SIZE*(i));
			end loop;
		else
			for i in C_NUM_REG downto 1 loop
				reg_inputs(C_SIZE*i -1 downto C_SIZE*(i-1)) <= data_i(C_SIZE*i -1 downto C_SIZE*(i-1));
			end loop;  
		end if;
	end process MUX_PROC;
	
	REG_GEN: for i in C_NUM_REG downto 1 generate
		REG : entity paralell_shift
			port map(
				clk => clk,
				data_i => reg_inputs(i*8-1 downto (i-1)*8),
				write => write_i,
				data_o => reg_outputs(i*8-1 downto (i-1)*8)	
				);
	end generate;

end RTL;
