-------------------------------------------------------------------------------
--
-- Title       : register_macroPO
-- Design      : parallel_registers
-- Author      : Vegard Endresen
-- Company     : NTNU
-- Description : Register macro, parallel output (no shift output)
-------------------------------------------------------------------------------

library IEEE;
use IEEE.STD_LOGIC_1164.all;

entity register_macroPO is 
	generic(
		C_SIZE : natural :=8;
		C_NUM_REG : natural := 4
	);
	port(	   
		clk		: in  std_logic;
		shift_i : in  std_logic_vector(C_SIZE-1 downto 0);
		shift	: in  std_logic;
		data_o	: out std_logic_vector(C_SIZE*C_NUM_REG -1 downto 0)
	);
	
	
end register_macroPO;


architecture RTL of register_macroPO is
	 
	
signal reg_inputs 	: std_logic_vector(C_SIZE*C_NUM_REG -1 downto 0);  
signal reg_outputs 	: std_logic_vector(C_SIZE*C_NUM_REG -1 downto 0);  
signal write_i : std_logic;

begin 		 
	
	write_i <= shift;
	
	OUT_PROC : process(reg_outputs,shift_i)
	begin		
		data_o <= reg_outputs;
		reg_inputs(C_SIZE*C_NUM_REG -1 downto C_SIZE*(C_NUM_REG-1)) <=  shift_i;
			for i in C_NUM_REG-1 downto 1 loop
				reg_inputs(C_SIZE*i -1 downto C_SIZE*(i-1)) <= reg_outputs(C_SIZE*(i+1) -1 downto C_SIZE*(i));
			end loop;
		
	end process OUT_PROC;

	
	REG_GEN: for i in C_NUM_REG downto 1 generate
		REG : entity paralell_shift
			port map(
				clk => clk,
				data_i => reg_inputs(i*8-1 downto (i-1)*8),
				write => shift,
				data_o => reg_outputs(i*8-1 downto (i-1)*8)	
				);
	end generate;

	

end RTL;
