-------------------------------------------------------------------------------
--
-- Title       : testbench
-- Design      : addressable_architecture
-- Author      : Vegard
-- Company     : NTNU
-- Description : 
-------------------------------------------------------------------------------

library IEEE;
use IEEE.STD_LOGIC_1164.all;
use IEEE.std_logic_unsigned.all;


entity testbench is
end testbench;

architecture testbench of testbench is

constant C_INPUT_SIZE : natural := 8;  
constant C_ADDR_SIZE : natural := 4;
constant C_NUM_REG 	: natural :=4;

signal a_i,b_i,c_i,d_i,a_o,b_o,c_o,d_o :std_logic_vector(C_INPUT_SIZE -1 downto 0);
signal eData :std_logic_vector(C_INPUT_SIZE-1 downto 0);

signal enabeled : std_logic;
signal addr : std_logic_vector(C_ADDR_SIZE-1 downto 0);

signal input : std_logic_vector(C_NUM_REG*C_INPUT_SIZE -1 downto 0);
signal output : std_logic_vector(C_NUM_REG*C_INPUT_SIZE -1 downto 0);

signal multiplexer_o : std_logic_vector(C_INPUT_SIZE -1 downto 0);



signal  clk 		:  std_logic;
signal	start		:  std_logic;
signal	count		:  std_logic_vector(C_ADDR_SIZE-1 downto 0);
signal	offset		:  std_logic_vector(C_ADDR_SIZE-1 downto 0);
signal	address		:  std_logic_vector(C_ADDR_SIZE-1 downto 0);
signal	complete	:  std_logic;


signal CPU_interrupt		:   std_logic; 
signal CPU_write			:   std_logic;
signal CPU_read				:   std_logic;
signal CPU_run				:   std_logic;
signal processing_complete	:   std_logic;
signal mode 				:   std_logic; 
signal run					:   std_logic;

signal OPC : std_logic;

begin	
	
-----------------------------------------------------------------------
---- Decoder 
-- 	input	<= d_i&c_i&b_i&a_i;
--	a_o <= output(C_INPUT_SIZE -1 downto 0);
--	b_o <= output(2*C_INPUT_SIZE -1 downto C_INPUT_SIZE);
--	c_o <= output(3*C_INPUT_SIZE -1 downto 2*C_INPUT_SIZE);
--	d_o <= output(4*C_INPUT_SIZE -1 downto 3*C_INPUT_SIZE);
--	
--	DECODER : entity decoder 
--		generic map(
--			C_INPUT_SIZE => C_INPUT_SIZE,
--			C_NOI => 4,
--			C_ADDR_SIZE => C_ADDR_SIZE
--		)
--		port map(
--			enabeled_n => enabeled,	
--			input => input,
--			output => output,
--			addr => addr,
--			external_data => eData
--		);
--		
--	STIMULI: process is
--	begin
--		enabeled <='0';
--		eData <= x"AB";
--		a_i <= x"12";
--		b_i <= x"23";
--		c_i <= x"DF";
--		d_i <= x"FE";
--		addr <= "00";	
--		for i in 1 to 3 loop
--			wait for 20 ns;
--			addr <= addr+1;
--		end loop;	
--		
--		
--		
--		for i in 1 to 3 loop
--			wait for 20 ns;
--			enabeled <='1';
--			addr <= addr+1;
--		end loop;	
--		
--		
--		wait;
--		
--	end process STIMULI;	 
-----------------------------------------------------------------------


-------------------------------------------------------------------------
---- Multiplexer 
--
--	input	<= d_i&c_i&b_i&a_i;
--	
--	MULTIPLEXER : entity multiplexer 
--	generic map(
--		C_INPUT_SIZE => C_INPUT_SIZE,
--		C_NOI => 4,
--		C_ADDR_SIZE => C_ADDR_SIZE
--	)
--	port map(
--		input => input,	
--		output => multiplexer_o,
--		addr => addr
--	);
--	
--	
--	STIMULI: process is
--	begin
--		
--		a_i <= x"12";
--		b_i <= x"23";
--		c_i <= x"DF";
--		d_i <= x"FE";
--		addr <= "00";	
--		for i in 1 to 3 loop
--			wait for 20 ns;
--			addr <= addr+1;
--		end loop;	
--		wait;
--		
--	end process STIMULI;
-------------------------------------------------------------------------


-------------------------------------------------------------------------
---- Address counter
--
--	CLKGEN: process is
--	begin
--		clk <='1';
--		wait for 10 ns;
--		clk <='0';
--		wait for 10 ns;
--		
--	end process;
--
--
--	ADDRESS_COUNTER : entity addr_counter 
--	generic map(
--		C_ADDR_SIZE => C_ADDR_SIZE
--	)
--	port map(
--		clk		=> clk,
--		start	=> start,
--		count	=> count,
--		offset	=> offset,
--		address	=> address,
--		complete => complete
--	);
--	
--	
--	STIMULI: process is
--	begin
--		start <='0';
--		count <=x"0";
--		offset <=x"8";
--		wait for 20 ns;
--		start <='1';
--		while complete ='0' loop
--			wait for 20 ns;
--		end loop;
--		start <='0';
--		wait;
--	end process STIMULI;
-------------------------------------------------------------------------

-----------------------------------------------------------------------
-- Address controller
	CLKGEN: process is
	begin
		clk <='1';
		wait for 10 ns;
		clk <='0';
		wait for 10 ns;
		
	end process;


	ADDRESS_CONTROLLER : entity controller 
	generic map(
		C_ADDR_SIZE => C_ADDR_SIZE
	)
	port map(
	 	clk => clk,
		CPU_interrupt	=> CPU_interrupt, 
		CPU_write => CPU_write,
	 	CPU_read => CPU_read,
		CPU_run =>CPU_run,	
		processing_complete	=> processing_complete,
		count 	=> count,
		offset 	=> offset,
		address => address,
		mode => mode,
		run	=> run,
		operation_complete => OPC
	);
	
	
	STIMULI: process is
	begin
		CPU_interrupt<='0'; 
		CPU_write <= '0';
	 	CPU_read <= '0';
		CPU_run <='0'; 
		count <=x"0";
		offset <=x"0";
		wait for 30 ns;	
		CPU_write <= '1';
		count <=x"3";
		offset <=x"8";
		while OPC ='0' loop
			wait for 20 ns;
		end loop;
		
		CPU_interrupt<='1'; 
		CPU_write <= '0';
	 	CPU_read <= '0';
		CPU_run <='0'; 
		
		wait;
	end process STIMULI;
-----------------------------------------------------------------------
	
	
end testbench;
