-------------------------------------------------------------------------------
--
-- Title       : addr_testbench
-- Design      : addressable_architecture
-- Author      : Vegard
-- Company     : NTNU
-- Description : 
-------------------------------------------------------------------------------

library IEEE;
use IEEE.STD_LOGIC_1164.all;
use IEEE.std_logic_unsigned.all;

entity addr_testbench is
end addr_testbench;


architecture addr_testbench of addr_testbench is


constant	C_REG_SIZE    		: natural	:=32;
constant	C_BIT_WIDTH			: natural   :=8;
constant	C_LOG_OPERAND_SIZE	: natural   :=5;
constant	C_ADDR_SIZE			: natural   :=5;
constant 	T_CLK				: time := 10 ns;


signal clk		:   std_logic;
signal run		:   std_logic;
signal write	:   std_logic;
signal addr		:   std_logic_vector(C_ADDR_SIZE-1 downto 0);
signal data_i	:	std_logic_vector(C_BIT_WIDTH-1 downto 0);
signal data_o	:   std_logic_vector(C_BIT_WIDTH-1 downto 0);
signal HW_complete	:  std_logic; 

signal CPU_interrupt		:   std_logic; 
signal CPU_write			:   std_logic;
signal CPU_read				:   std_logic;
signal CPU_run				:   std_logic;
signal count				:  std_logic_vector(C_ADDR_SIZE-1 downto 0);	
signal offset				:  std_logic_vector(C_ADDR_SIZE-1 downto 0);

signal address : std_logic_vector(C_ADDR_SIZE-1 downto 0);

signal OPC : std_logic;


signal dataI, dataO : std_logic_vector(C_REG_SIZE-1 downto 0);

signal writeI,shift : std_logic;

begin
	
	
	MULTIPLIER : entity sequential_multiplier
		generic map(
		    C_REG_SIZE    		=> C_REG_SIZE,
			C_BIT_WIDTH			=> C_BIT_WIDTH,
			C_LOG_OPERAND_SIZE	=> C_LOG_OPERAND_SIZE,
			C_ADDR_SIZE			=> C_ADDR_SIZE
		)
		port map(
			clk => clk,
			run => run,
			write => write,
			addr => address,
			data_i => data_i,
			data_o => data_o,
			HW_complete => HW_complete
	);
	
	CLKGEN : process is 
	 begin
		 clk <= '1';
		wait for T_CLK/2;
		clk <='0';
		wait for T_CLK/2;
	end process CLKGEN;
	
	
	ADDRESS_CONTROLLER : entity controller 
	generic map(
		C_ADDR_SIZE => C_ADDR_SIZE
	)
	port map(
	 	clk => clk,
		CPU_interrupt	=> CPU_interrupt, 
		CPU_write => CPU_write,
	 	CPU_read => CPU_read,
		CPU_run =>CPU_run,	
		processing_complete	=> HW_complete,
		count 	=> count,
		offset 	=> offset,
		address => address,
		mode => write,
		read => shift,
		run	=> run,
		operation_complete => OPC
	);
	
	
	DATA_IN : entity register_macroPI  	  
	generic map(C_SIZE => C_REG_SIZE/4, C_NUM_REG =>4)
	port map(
		data_i 		=> dataI,
		write	 	=> writeI,
		clk			=> clk,
		shift 		=> write,
		shift_o		=> data_i
	
	);	   	
	
	DATA_OUT : entity register_macroPO
	generic map(C_SIZE => C_REG_SIZE/4, C_NUM_REG =>4)
	port map(
	   	clk 	=> clk, 
		shift 	=> shift,
		shift_i => data_o,
		data_o => dataO
	);
	
	
	
	
----------------------------------------------
--  Benchmark stimulus
--
--	STIMULUS : process is 
--	begin
--		run <= '0';
--		write <='1';
--		addr <= (others =>'0');
--		data_i <= (others =>'1');
--		wait for T_CLK;
--		for i in 0 to 8 loop
--			wait for T_CLK;
--			addr <= addr+1;
--		end loop;
--		data_i <= (others =>'0');
--		for i in 9 to 15 loop
--			wait for T_CLK;
--			addr <= addr+1;
--		end loop;
--		write <='0';
--		addr <=(others =>'0');
--		wait for T_CLK;
--		run <='1';
--		wait;
--		
--	end process STIMULUS;


	STIMULI: process is
	begin
		CPU_interrupt<='0'; 
		CPU_write <= '0';
	 	CPU_read <= '0';
		CPU_run <='0'; 
		count <="00100";
		offset <=(others =>'0');
		dataI <=x"00abcdef";
		writeI <='1';
		wait for T_CLK;
		writeI<='0';
		CPU_write <= '1';
		while OPC ='0' loop
			wait for T_CLK;
		end loop;
		
		CPU_interrupt<='1'; 
		CPU_write <= '0';
	 	CPU_read <= '0';
		CPU_run <='0'; 
		dataI <=x"abcdef12";
		offset <="00100";
		writeI <='1';
		wait for T_CLK;		
		writeI <='0';
		CPU_interrupt <='0';
		CPU_write <= '1';
		while OPC ='0' loop
			wait for T_CLK;
		end loop;
		
		CPU_interrupt<='1'; 
		CPU_write <= '0';
	 	CPU_read <= '0';
		CPU_run <='0';
		offset <="01000";
		dataI <=x"a2000000";
		writeI <='1';
		wait for T_CLK;
		writeI<='0';
		CPU_interrupt<='0';
		CPU_write <= '1';
		while OPC ='0' loop
			wait for T_CLK;
		end loop;
		CPU_interrupt<='1'; 
		CPU_write <= '0';
	 	CPU_read <= '0';
		CPU_run <='0';
		offset <="01100";
		dataI <=x"5bf134f0";
		writeI <='1';
		wait for T_CLK;
		writeI<='0';
		CPU_interrupt<='0';
		CPU_write <= '1';
		while OPC ='0' loop
			wait for T_CLK;
		end loop;
		CPU_interrupt<='1'; 
		CPU_write <= '0';
	 	CPU_read <= '0';
		CPU_run <='0';
		offset <="10000";
		dataI <=x"00000026";
		count <="00001";
		writeI <='1';
		wait for T_CLK;
		writeI<='0';
		CPU_interrupt<='0'; 
		CPU_write <= '1';
		while OPC ='0' loop
			wait for T_CLK;
		end loop;
		CPU_interrupt<='1'; 
		CPU_write <= '0';
	 	CPU_read <= '0';
		CPU_run <='0';
		offset <= (others =>'1'); -- should not be neccessary
		wait for T_CLK;
		CPU_interrupt<='0';
		CPU_run <='1';
		while HW_complete ='0' loop
			wait for T_CLK;
		end loop;
			
		CPU_interrupt<='1'; 
		CPU_write <= '0';
	 	CPU_read <= '0';
		CPU_run <='0';
		offset <="01000";
		count <="01000";
		wait for T_CLK;
		CPU_interrupt<='0'; 
		CPU_write <= '0';
	 	CPU_read <= '1';
		CPU_run <='0';
		while OPC ='0' loop
			wait for T_CLK;
		end loop;
		 wait;	
		
	end process STIMULI;


end addr_testbench;
