-------------------------------------------------------------------------------
--
-- Title       : SPI_slave_test
-- Design      : reconfiguration
-- Author      : Vegard
-- Company     : NTNU
-- Description : 
-------------------------------------------------------------------------------


library IEEE;
use IEEE.STD_LOGIC_1164.all;

entity shift_chain_test is
	
end shift_chain_test;



architecture arch of shift_chain_test is

constant CPU_REG_SIZE : natural :=32;
constant SHIFT_SIZE : natural :=8;	  
constant T_CLK		: time :=10ns;


signal clk 			: std_logic;		
signal run			: std_logic;
signal completion	: std_logic;

signal shift		: std_logic;
signal shift_o, shift_i 		: std_logic_vector(SHIFT_SIZE-1 downto 0);


signal reset : std_logic;
signal write : std_logic; 
signal dataI : std_logic_vector(CPU_REG_SIZE -1 downto 0);
signal dataO : std_logic_vector(CPU_REG_SIZE -1 downto 0);

signal count : std_logic_vector(4 downto 0);

signal CPU_shift : std_logic;
signal CPU_run 	 : std_logic;
signal CPU_interrupt	 : std_logic;

signal controller_waiting : std_logic;


begin
	
	-- Clock generation
	CLKGEN: process is
	begin
		clk <= '1';
		wait for T_CLK/2;
		clk <= '0';
		wait for T_CLK/2;
	end process;
	
	
	DATA_IN : entity register_macroPI  	  
	generic map(C_SIZE => SHIFT_SIZE, C_NUM_REG =>CPU_REG_SIZE/SHIFT_SIZE)
	port map(
		data_i 		=> dataI,
		write	 	=> write,
		clk			=> clk,
		shift 		=> shift,
		shift_o		=> shift_i
	
	);	   	
	
	DATA_OUT : entity register_macroPO
	generic map(C_SIZE => SHIFT_SIZE, C_NUM_REG =>CPU_REG_SIZE/SHIFT_SIZE)
	port map(
	   	clk 	=> clk, 
		shift 	=> shift,
		shift_i => shift_o,
		data_o => dataO
	);
	
	SERIAL_CONTROLLER : entity serial_controller
	port map(
		clk 			=> clk,	 
		CPU_interrupt	=> CPU_interrupt,
		CPU_shift 	=> CPU_shift,
		CPU_run 	=> CPU_run,
		count 		=> count,
		shift		=> shift,
		HW_complete => completion,
		controller_waiting => controller_waiting,
		run => run
		);
		
-----------------------------------------------------------
---- State machine test
--
--	RECONF : entity shift_chain 
--	port map(
--		clk		=> clk, 
--		run 	=> run,	
--		shift 	=> shift,
--		shift_i => shift_i,
--		shift_o => shift_o,
--		completion => completion
--	); 
--
--		
--	INPUT_PROC : process is
--	begin
--		
--		write <='1';
--		count <="01111";
--		dataI <= x"FBC1";
--		CPU_interrupt <= '0'; 
--		CPU_shift <='0';
--		CPU_run <='0';
--		wait for 20 ns;	 
--		write <='0';
--		CPU_shift <='1';
--		while completion ='0' loop
--			wait for 20 ns;
--		end loop;
--		CPU_interrupt <='1';
--		CPU_shift <='0';
--		CPU_run   <='0';
--		wait for 20 ns;
--		CPU_interrupt <='0';
--		CPU_shift <='0';
--		CPU_run   <='1';
--		wait for 60 ns;
--		CPU_shift <='0';
--		CPU_run   <='0';
--		CPU_interrupt <='1';
--		wait for 20 ns;
--		dataI <= x"C00C";
--		write <= '1';
--		wait for 20 ns;
--		write <='0';
--		wait for 20 ns;
--		CPU_interrupt <='0';
--		CPU_shift <='1';
--		CPU_run   <='0';
--		wait;
--		
--	end process INPUT_PROC;
---------------------------------------------------------


---------------------------------------------------------
-- Sequential multiplicator test

	MULT : entity multiplier
	generic map(OPERAND_SIZE => 32,
	LOG_OPERAND_SIZE => 5
	)
	port map(
		clk => clk,
		reset => reset,
		shift_i => shift_i,
		shift_o => shift_o,
		shift_g => shift,
		run 	 => run,
		finish => completion
		
		
	);


	INPUT : process is
	begin		   
		
		reset <='0';
		write <='1';
		count <="00011";
		dataI <= x"A2000000";
		CPU_interrupt <= '0'; 
		CPU_shift <='0';
		CPU_run <='0';
		wait for T_CLK; 
		write <='0';
		CPU_shift <='1';
		while controller_waiting ='0' loop
			wait for T_CLK;	 
		end loop;
		CPU_interrupt <='1';
		CPU_shift <='0';
		CPU_run   <='0';  
		dataI <= x"5bf134f0";
		write <='1';
		wait for T_CLK; 
		write <='0';
			CPU_interrupt <='0';
		CPU_shift <='1';
		CPU_run   <='0';  
		while controller_waiting ='0' loop
			wait for T_CLK;	 
		end loop;	 
		CPU_interrupt <='1';
		CPU_shift <='0';
		CPU_run   <='0';  
		dataI <= x"0000004C";
		count <="00000";
		write <='1';
		wait for T_CLK ;	
		write <='0';
		CPU_interrupt <='0';
		CPU_shift <='1';
		while controller_waiting ='0' loop
			wait for T_CLK;	 
		end loop;
		CPU_interrupt <='1';
		CPU_shift <='0';
		CPU_run   <='0'; 
		count <="00011";
		dataI <= x"ABCDEF12";
		write <='1';
		wait for T_CLK ;
		CPU_interrupt <='0';
		CPU_shift <='1';   
		write <='0';
		while controller_waiting ='0' loop
			wait for T_CLK;	 
		end loop;
		CPU_interrupt <='1';
		CPU_shift <='0';
		CPU_run   <='0';
		dataI <= x"00abcdef"; 
		write <='1';
		wait for T_CLK;
		write <='0';
		CPU_interrupt <='0';
		CPU_shift <='1';
		CPU_run   <='0'; 
		while controller_waiting ='0' loop
			wait for T_CLK;	 
		end loop;
		CPU_interrupt <='1';
		CPU_shift <='0';
		CPU_run   <='0';
		wait for T_CLK;
		CPU_interrupt <='0';
		CPU_shift <='0';
		CPU_run   <='1';
		wait;
		
	end process INPUT;
---------------------------------------------------------
	

end arch;
