-------------------------------------------------------------------------------
--
-- Title       : register_testbench
-- Design      : serial_shift_registers
-- Author      : Vegard	Endresen
-- Company     : NTNU
-- Description : Testbench for serial shift registers. DUT uncommented
-------------------------------------------------------------------------------

library IEEE;
use IEEE.STD_LOGIC_1164.all;

entity register_testbench is
end register_testbench;

architecture TEST of register_testbench is

-- Common constants and signals, all fields are not used
-- for each component though

constant C_SIZE 	: natural :=8;
constant C_GSIZE 	: natural :=4;
constant T_CLK  	: time :=10 ns;

signal clk 		: std_logic;
signal shift	: std_logic; 
signal write	: std_logic;
signal shift_i	: std_logic;
signal shift_o	: std_logic;
signal dataI	: std_logic_vector(C_SIZE-1 downto 0);
signal dataO 	: std_logic_vector(C_SIZE-1 downto 0);

begin
	
	
  -- Clock generation
  CLKGEN: process is
  begin
    clk <= '1';
    wait for T_CLK/2;
    clk <= '0';
    wait for T_CLK/2;
  end process;
  
-----------------------------------------------------
-- shift_register

	REG : entity shift_register
	generic map(C_SIZE => C_SIZE) 
	port map(
      
	    clk 		=> clk, 
		par_i 		=> dataI,
		par_write	=> write,
		shift 		=> shift,
		shift_i 	=> shift_i,
		shift_o 	=> shift_o,
		par_o		=> dataO
    );
		

  STIMULI: process is
  begin		   
	  	dataI 	<= (others =>'0');
		write 	<='0';
		shift 	<='0'; 
		shift_i <='0';
		dataI <=x"AB";
		wait for T_CLK+T_CLK/2;
 		shift <='1';
		for i in 0 to C_SIZE-1 loop
			shift_i<=dataI(i);
			wait for T_CLK;
		end loop;
		
		shift <='0';
		dataI <=x"BA";
		write <='1';  
		wait for T_CLK;
		write <='0';
		wait;
	  
  end process STIMULI;	
--------------------------------------------------- 
 
----------------------------------------------------- 
---- shift_registerSI	  
--
--	REG_SI: entity shift_registerSI
--	generic map( C_SIZE => C_SIZE)
-- 	port map(
--		clk 	=> clk,  
--		shift 	=> shift,
--		shift_i => shift_i,
--		shift_o => shift_o,
--		par_o 	=> dataO
--		);
--  
--
--
--	STIMULI: process is
--	begin		   
--	  	dataI 	<= (others =>'0');
--		write 	<='0';
--		shift 	<='0'; 
--		shift_i <='0';
--		dataI <=x"AB";
--		wait for T_CLK+T_CLK/2;
-- 		shift <='1';
--		for i in 0 to C_SIZE-1 loop
--			shift_i<=dataI(i);
--			wait for T_CLK;
--		end loop;
--		shift <='0';
--		wait;
--	  
--  end process STIMULI;	
----------------------------------------------------- 

--------------------------------------------
---- shift_registerSIPO 
--
--  	REG_SIPO : entity shift_registerSIPO
--  	generic map( C_SIZE => C_SIZE)
--  	port map(
--		clk		=> clk, 
--		shift	=> shift,
--		shift_i	=> shift_i,
--		par_o 	=> dataO
--		);
--  
--	STIMULI: process is
--	begin		   
--	  	dataI 	<= (others =>'0');
--		write 	<='0';
--		shift 	<='0'; 
--		shift_i <='0';
--		dataI <=x"AB";
--		wait for T_CLK+T_CLK/2;
-- 		shift <='1';
--		for i in 0 to C_SIZE-1 loop
--			shift_i<=dataI(i);
--			wait for T_CLK;
--		end loop;
--		shift <='0';
--		wait;
--	  
--  end process STIMULI;	
-------------------------------------------- 


--------------------------------------------
---- shift_registerPISO 
--
--  	REG_PISO : entity shift_registerPISO
--  	generic map( C_SIZE => C_SIZE)
--  	port map(
--		clk			=>clk, 
--		par_i 		=> dataI,
--		par_write 	=> write,
--		shift		=> shift,
--		shift_o	 	=> shift_o
--		);
--  
--	STIMULI: process is
--	begin		   
--	  	dataI 	<= (others =>'0');
--		write 	<='0';
--		shift 	<='0'; 
--		dataI <=x"AB";
--		wait for T_CLK+T_CLK/2;
-- 		write <='1'; 
--		wait for T_CLK+T_CLK/2;
--		write <='0';
--		shift <='1';
--		for i in 0 to C_SIZE-1 loop
--			wait for T_CLK;
--		end loop;
--		shift <='0';
--		wait;
--	  
--  end process STIMULI;	
--------------------------------------------

--------------------------------------------
---- shift_registerGI 
--
--  	REG_GI : entity shift_registerPIG
--  	generic map( 
--	  C_SIZE => C_SIZE,
--	  C_INPUT_SIZE => C_GSIZE
--	  )
--  	port map(
--		clk			=>clk, 
--		par_i 		=> dataI(C_SIZE-1 downto C_GSIZE),
--		par_write 	=> write,
--		shift		=> shift,
--		shift_o	 	=> shift_o
--		);
--  
--	STIMULI: process is
--	begin		   
--	  	dataI <= (others =>'0');
--		write <='0';
--		shift <='0'; 
--		dataI <=x"AB";
--		wait for T_CLK+T_CLK/2;
-- 		write <='1';
--		wait for T_CLK;
--		write <='0';
--		shift <='1';
--		for i in 0 to C_SIZE-1 loop
--			shift_i <= dataI(i);
--			wait for T_CLK;
--		end loop;	 
--		
--		shift <='0';
--		write <='1';
--		wait for T_CLK;
--		write <='0';
--		wait;
--	  
--  end process STIMULI;	
--------------------------------------------




end TEST;
