-------------------------------------------------------------------------------
--
-- Title       : serial
-- Design      : tcl
-- Author      : Vegard
-- Company     : NTNU
-- Description : 
--
-------------------------------------------------------------------------------

library IEEE;
use IEEE.STD_LOGIC_1164.all;

entity serial is
	generic(
		constant C_NUM_REG : natural := 8;	 
		constant C_REG_SIZE : natural :=1
	);
	port(
		clk			: in  std_logic;
		run			: in  std_logic;
		shift		: in  std_logic;
		shift_i		: in  std_logic;
		shift_o		: out std_logic;
		HW_complete	: out std_logic;
		run_short	: out std_logic;
		local_short	: out std_logic_vector(C_NUM_REG*C_REG_SIZE-1 downto 0)
	);
end serial;


architecture serial of serial is



signal local_i 	: std_logic_vector(C_NUM_REG*C_REG_SIZE-1 downto 0);
signal shift_il	: std_logic_vector(C_NUM_REG-1 downto 0);
signal shift_ol	: std_logic_vector(C_NUM_REG-1 downto 0);

begin
	
	HW_complete <='0';
	
 	SHIFT_PROC : process(shift_ol,shift_i) is
	begin		  
		shift_il(0) <= shift_i;
		for i in 0 to C_NUM_REG-2 loop
			shift_il(i+1) <= shift_ol(i);
		end loop;
		shift_o <= shift_ol(C_NUM_REG-1);
	end process SHIFT_PROC;
	
	REG_GEN: for i in 0 to C_NUM_REG-1 generate
		DATA_OUT : entity shift_registerSI
		generic map(C_SIZE => C_REG_SIZE)
		port map(
		    clk 	=> clk, 
			shift 	=> shift,
			shift_i => shift_il(i),	
			shift_o => shift_ol(i),
			par_o	 =>  local_i((i+1)*C_REG_SIZE-1 downto (i)*C_REG_SIZE)	
		);
	end generate;
	
	LOCAL_PROC : process(local_i,run) is
	begin
		run_short <= run;
		local_short <= local_i;
	
	end process LOCAL_PROC;
	
	
	

end serial;
