library IEEE;
use IEEE.STD_LOGIC_1164.all;
entity serial_sm is
	port(
 		clk			: in  std_logic;
		run			: in  std_logic;
		shift		: in  std_logic;
		shift_i		: in  std_logic;
		shift_o		: out std_logic;
		HW_complete	: out std_logic
	);end serial_sm;

architecture arch of serial_sm is
------------------------------------------------------------------
-- SM REG
constant C_STATE_SIZE 	: natural := 5;
signal nstate 			: std_logic_vector(C_STATE_SIZE-1 downto 0);
signal cstate 			: std_logic_vector(C_STATE_SIZE-1 downto 0);
signal state_write 		: std_logic;
--------------------------------------------------------------------
-- STATE CONSTANTS
constant	s0	 : std_logic_vector(C_STATE_SIZE-1 downto 0) := "00000";
constant	s1	 : std_logic_vector(C_STATE_SIZE-1 downto 0) := "00001";
constant	s2	 : std_logic_vector(C_STATE_SIZE-1 downto 0) := "00010";
constant	s3	 : std_logic_vector(C_STATE_SIZE-1 downto 0) := "00011";
constant	s4	 : std_logic_vector(C_STATE_SIZE-1 downto 0) := "00100";
constant	s5	 : std_logic_vector(C_STATE_SIZE-1 downto 0) := "00101";
constant	s6	 : std_logic_vector(C_STATE_SIZE-1 downto 0) := "00110";
constant	s7	 : std_logic_vector(C_STATE_SIZE-1 downto 0) := "00111";
constant	s8	 : std_logic_vector(C_STATE_SIZE-1 downto 0) := "01000";
constant	s9	 : std_logic_vector(C_STATE_SIZE-1 downto 0) := "01001";
constant	s10	 : std_logic_vector(C_STATE_SIZE-1 downto 0) := "01010";
constant	s11	 : std_logic_vector(C_STATE_SIZE-1 downto 0) := "01011";
constant	s12	 : std_logic_vector(C_STATE_SIZE-1 downto 0) := "01100";
constant	s13	 : std_logic_vector(C_STATE_SIZE-1 downto 0) := "01101";
constant	s14	 : std_logic_vector(C_STATE_SIZE-1 downto 0) := "01110";
constant	s15	 : std_logic_vector(C_STATE_SIZE-1 downto 0) := "01111";
constant	s16	 : std_logic_vector(C_STATE_SIZE-1 downto 0) := "10000";
constant	s17	 : std_logic_vector(C_STATE_SIZE-1 downto 0) := "10001";
constant	s18	 : std_logic_vector(C_STATE_SIZE-1 downto 0) := "10010";
constant	s19	 : std_logic_vector(C_STATE_SIZE-1 downto 0) := "10011";
constant	s20	 : std_logic_vector(C_STATE_SIZE-1 downto 0) := "10100";
constant	s21	 : std_logic_vector(C_STATE_SIZE-1 downto 0) := "10101";
constant	s22	 : std_logic_vector(C_STATE_SIZE-1 downto 0) := "10110";
constant	s23	 : std_logic_vector(C_STATE_SIZE-1 downto 0) := "10111";
constant	s24	 : std_logic_vector(C_STATE_SIZE-1 downto 0) := "11000";
constant	s25	 : std_logic_vector(C_STATE_SIZE-1 downto 0) := "11001";
constant	s26	 : std_logic_vector(C_STATE_SIZE-1 downto 0) := "11010";
constant	s27	 : std_logic_vector(C_STATE_SIZE-1 downto 0) := "11011";
constant	s28	 : std_logic_vector(C_STATE_SIZE-1 downto 0) := "11100";
constant	s29	 : std_logic_vector(C_STATE_SIZE-1 downto 0) := "11101";
constant	s30	 : std_logic_vector(C_STATE_SIZE-1 downto 0) := "11110";
constant	s31	 : std_logic_vector(C_STATE_SIZE-1 downto 0) := "11111";
begin
--------------------------------------------------------------------
-- SHIFT CHAIN

STATE_REG : entity shift_register
generic map(C_SIZE => C_STATE_SIZE)
port map(	clk			=> clk,
par_i 		=> nstate,
par_write 	=> state_write,
shift 		=> shift,
shift_i 	=> shift_i,
shift_o 	=> shift_o,
par_o		=> cstate
);
--------------------------------------------------------------------
WRITE_PROC : process(run,cstate) is
begin
if run ='0' then
state_write		<='0';
nstate			<=s0;
else 
state_write 	<='1';
--------------------------------------------
-- state machine code
case cstate is
when s0 =>
nstate <= s1;
when s1 =>
nstate <= s2;
when s2 =>
nstate <= s3;
when s3 =>
nstate <= s4;
when s4 =>
nstate <= s5;
when s5 =>
nstate <= s6;
when s6 =>
nstate <= s7;
when s7 =>
nstate <= s8;
when s8 =>
nstate <= s9;
when s9 =>
nstate <= s10;
when s10 =>
nstate <= s11;
when s11 =>
nstate <= s12;
when s12 =>
nstate <= s13;
when s13 =>
nstate <= s14;
when s14 =>
nstate <= s15;
when s15 =>
nstate <= s16;
when s16 =>
nstate <= s17;
when s17 =>
nstate <= s18;
when s18 =>
nstate <= s19;
when s19 =>
nstate <= s20;
when s20 =>
nstate <= s21;
when s21 =>
nstate <= s22;
when s22 =>
nstate <= s23;
when s23 =>
nstate <= s24;
when s24 =>
nstate <= s25;
when s25 =>
nstate <= s26;
when s26 =>
nstate <= s27;
when s27 =>
nstate <= s28;
when s28 =>
nstate <= s29;
when s29 =>
nstate <= s30;
when s30 =>
nstate <= s31;
when s31 =>
nstate <= s0;
when others =>

end case;
--------------------------------------------
end if;
end process;
HW_COMPLETE_PROC : process(cstate) is
begin
	if cstate = s15 then
HW_complete <= '1';
else
	HW_complete <='0';
end if;
end process HW_COMPLETE_PROC;
end arch;
