-------------------------------------------------------------------------------
--
-- Title       : shift_chain_test
-- Design      : reconfiguration
-- Author      : Vegard Endresen
-- Company     : NTNU
-- Description : 
-------------------------------------------------------------------------------


library IEEE;
use IEEE.STD_LOGIC_1164.all;

entity testbench is
	
end testbench;



architecture TEST of testbench is

constant CPU_REG_SIZE : natural :=32;
constant T_CLK		  : time := 10 ns;
constant COUNT_SIZE	  : natural := 5;

signal write 			:  std_logic;
signal CPU_shift 		:  std_logic;
signal CPU_run 	 		:  std_logic;
signal CPU_interrupt	:  std_logic;
signal dataI			:  std_logic_vector(CPU_REG_SIZE-1 downto 0);
signal dataO			:  std_logic_vector(CPU_REG_SIZE-1 downto 0);



signal clk 			: std_logic;		
signal run			: std_logic;
signal HW_complete	: std_logic;

signal shift		: std_logic;
signal shift_o		: std_logic;
signal shift_i 		: std_logic;




signal count : std_logic_vector(COUNT_SIZE-1 downto 0);
signal controller_waiting : std_logic;


begin
	
	-- Clock generation
	CLKGEN: process is
	begin
		clk <= '1';
		wait for T_CLK/2;
		clk <= '0';
		wait for T_CLK/2;
	end process;
--	
	
	DATA_IN : entity shift_registerPISO
	generic map(C_SIZE => CPU_REG_SIZE)
	port map(
		par_i 		=> dataI,
		par_write 	=> write,
		clk			=> clk,
		shift 		=> shift,
		shift_o		=> shift_i
	
	);	   	
	
	DATA_OUT : entity shift_registerSIPO
	generic map(C_SIZE => CPU_REG_SIZE)
	port map(
	   	clk 	=> clk, 
		shift 	=> shift,
		shift_i => shift_o,
		par_o	 =>  dataO		
	);
	
	SERIAL_CONTROLLER : entity serial_controller
	generic map(C_COUNT_SIZE => COUNT_SIZE)
	port map(
		clk 			=> clk,	 
		CPU_interrupt	=> CPU_interrupt,
		CPU_shift 		=> CPU_shift,
		CPU_run 		=> CPU_run,
		count 			=> count,
		shift			=> shift,
		HW_complete 	=> HW_complete,
		run 			=> run,
		controller_waiting => controller_waiting
		);
		
---------------------------------------------------------
---- State machine test
--
--	RECONF : entity sm_serial 
--	port map(
--		clk		=> clk, 
--		run 	=> run,	
--		shift 	=> shift,
--		shift_i => shift_i,
--		shift_o => shift_o,
--		HW_complete => HW_complete
--	); 
--
--		
--	INPUT_PROC : process is
--	begin
--		
--		write <='1';
--		count <="00011";
--		dataI <= x"A";
--		CPU_interrupt <= '0'; 
--		CPU_shift <='0';
--		CPU_run <='0';
--		wait for T_CLK;	 
--		write <='0';
--		CPU_shift <='1';
--		while controller_waiting ='0' loop
--			wait for T_CLK;	 
--		end loop;
--		CPU_interrupt <='1';
--		CPU_shift <='0';
--		CPU_run   <='0';
--		wait for T_CLK;	
--		CPU_interrupt <='0';
--		CPU_shift <='0';
--		CPU_run   <='1';
--		while HW_complete ='0' loop
--			wait for T_CLK;	 
--		end loop;
--		CPU_interrupt <= '1'; 
--		CPU_shift <='0';
--		CPU_run <='0';
--		wait for T_CLK;	 
--		write <='1';
--		dataI <= x"2";
--		wait for T_CLK;	 
--		write <='0'; 
--		dataI <= x"0";
--		CPU_interrupt <= '0'; 
--		CPU_shift <='1';
--		CPU_run <='0'; 
--		wait for T_CLK;
--		while controller_waiting ='0' loop
--			wait for T_CLK;	 
--		end loop;
--		
--		CPU_interrupt <= '1'; 
--		CPU_shift <='0';
--		CPU_run <='0';
--		wait for T_CLK;
--		CPU_interrupt <= '0'; 
--		CPU_shift <='0';
--		CPU_run <='1';		
--		
--		wait; 
--	end process INPUT_PROC;
-------------------------------------------------------


---------------------------------------------------------
-- Sequential multiplicator test   

-- computing ABCDEF12^2, continuing from interrupt 
--
--	  partial result : 0x5BF134F0A2000000
--	  cstate		 : 0x2
--	  count			 : 0x9
--    operandA		 : 0x00ABCDEF

	MULT : entity sequential_multiplier
	generic map(OPERAND_SIZE => CPU_REG_SIZE,
	LOG_OPERAND_SIZE => 5
	)
	port map(
		clk => clk,
		shift_i => shift_i,
		shift_o => shift_o,
		shift_g => shift,
		run 	 => run,
		HW_complete => HW_complete
	);


	INPUT : process is
	begin
		
		write <='1';
		count <="11111";
		dataI <= x"A2000000";
		CPU_interrupt <= '0'; 
		CPU_shift <='0';
		CPU_run <='0';
		wait for T_CLK;	 
		------------------------------------------
		-- Partial operand (0x5BF134F0A2000000)
		write <='0';
		CPU_shift <='1';
		while controller_waiting ='0' loop
			wait for T_CLK;
		end loop;
		
		CPU_interrupt <='1';
		CPU_shift <='0';
		CPU_run   <='0';
		dataI <= x"5bf134f0";
		write <='1';
		wait for T_CLK;	 
		write <='0';
		CPU_interrupt <='0';
		CPU_shift <='1';
		CPU_run   <='0'; 
		while controller_waiting ='0' loop
			wait for T_CLK;	
		end loop;
		--------------------------------------------
		
		--------------------------------------------
		--count & cstate (0x26)
	
		CPU_interrupt <='1';
		CPU_shift <='0';
		CPU_run   <='0'; 
		dataI <= x"00000026";
		write <='1';
		count <="00110";
		wait for T_CLK;	
		CPU_interrupt <='0';
		CPU_shift <='1';
		CPU_run   <='0';
		write <='0';
		while controller_waiting ='0' loop
			wait for T_CLK;
		end loop;
		--------------------------------------------
		--OperandA and OperandB (00ABCDEF,ABCDEF12) 
		CPU_interrupt <='1';
		CPU_shift <='0';
		CPU_run   <='0';
		write <='1';
		dataI <= x"ABCDEF12";
		count <="11111";
		wait for T_CLK;
		CPU_interrupt <='0';
		CPU_shift <='1';
		CPU_run   <='0';
		write <='0';
		while controller_waiting ='0' loop
			wait for T_CLK;
		end loop;
		CPU_interrupt <='1';
		CPU_shift <='0';
		CPU_run   <='0'; 
		dataI <= x"00ABCDEF";
		write <='1';
		wait for T_CLK ;
		write <='0';
		CPU_interrupt <='0';
		CPU_shift <='1';
		CPU_run   <='0'; 
		while controller_waiting ='0' loop
			wait for T_CLK;
		end loop;
		CPU_interrupt <='1';
		CPU_shift <='0';
		CPU_run   <='0';
		wait for T_CLK;
		--------------------------------------------
		-- Run 
		CPU_interrupt <='0';
		CPU_shift <='0';
		CPU_run   <='1';
		while HW_complete ='0' loop
			wait for T_CLK;
		end loop;
		wait;
		
	end process INPUT;
---------------------------------------------------------
	

end TEST;
