-------------------------------------------------------------------------------
--
-- Title       : shift-controller
-- Design      : serial_controller
-- Author      : Vegard	Endresen
-- Company     : NTNU
-- Description : Controller that will pull a shift signal high for count+1 
--				 positive clock edges with respect to the input count. Signals
--				 completion with a pulse in the last shift cycle.
-------------------------------------------------------------------------------

library IEEE;
use IEEE.STD_LOGIC_1164.all;
use IEEE.std_logic_unsigned.all;

entity controller is
	generic(C_COUNT_SIZE : natural :=5);
	port(
		clk				: in  std_logic;
		start			: in  std_logic;
		count			: in  std_logic_vector(C_COUNT_SIZE-1 downto 0);
		shift			: out std_logic;
		shift_complete	: out std_logic
		);
end controller;

architecture controller of controller is

	signal cnt 	: std_logic_vector(C_COUNT_SIZE-1 downto 0);
 	type state	is (IDLE,COUNTING);
	signal cstate, nstate : state;
	
begin
	
	NSTATE_PROC : process(cstate,start,cnt) is
	begin
		case cstate is
			when IDLE =>  
			if(start)='1' then
				nstate <= COUNTING;
			else
				nstate <= IDLE;
			end if;
			when COUNTING =>
				if(cnt = count) then
					nstate <= IDLE;
				else			  
					nstate <= COUNTING;
				end if;
			end case;	
	end process NSTATE_PROC;
	
	CSTATE_PROC : process(clk)is
	begin					   
		if clk'event and clk ='0' then
			cstate <= nstate;
		end if;
	end process CSTATE_PROC;
	
	SHIFT_PROC : process(cstate) is
	begin
		if cstate = IDLE then
			shift <='0';
		else
			shift <='1';	
		end if;
	end process SHIFT_PROC;
	
	COUNT_PROC : process(clk) is
	begin
		if(clk'event and clk ='0') then
			if cstate = IDLE then
				cnt <=(others => '0');
			else
				cnt <= cnt+1;
			end if;
		end if;
	end process COUNT_PROC;
	
	COMPLETION_PROC: process(cstate,cnt,count) is
	begin
		if cstate = COUNTING and cnt = count then
			shift_complete <='1';
		else
			shift_complete <='0';
		end if;
		
	end process COMPLETION_PROC;

end controller;













