-------------------------------------------------------------------------------
--
-- Title       : shift_register
-- Design      : serial_shift_registers
-- Author      : Vegard Endresen
-- Description : Serial shift register, parallel input generic only
-------------------------------------------------------------------------------

library IEEE;
use IEEE.STD_LOGIC_1164.all;

entity shift_registerPIG is
	generic(
		--register size
		C_SIZE			: natural :=8;
		C_INPUT_SIZE 	: natural :=4	
	);

	port(
				
		clk			: in  std_logic;  
		par_i   	: in  std_logic_vector(C_INPUT_SIZE-1 downto 0);
		par_write	: in  std_logic;
		shift		: in  std_logic;
		shift_o		: out std_logic;
		par_o		: out std_logic_vector(C_SIZE-1 downto 0)
	 );
	

end entity shift_registerPIG;
					  
architecture RTL  of shift_registerPIG is 

signal memory 	: std_logic_vector(C_SIZE-1 downto 0);

begin
	
	par_o 	<= memory;
	shift_o <= memory(0);
	
	WRITE_PROC : process(clk) is
	begin
	if(clk'event and clk ='1') then
		if shift ='1' then
			for i in C_SIZE-2 downto 0 loop
				memory(i) <= memory(i+1);
			end loop;
			memory(C_SIZE-1) <= '0';
		elsif par_write ='1' then
			memory(C_SIZE-1 downto (C_SIZE-C_INPUT_SIZE) ) <= par_i; 
		else	
			memory <= memory;	
		end if;
	end if;
	end process WRITE_PROC; 
								
end architecture RTL ;
