-------------------------------------------------------------------------------
--
-- Title       : shift_register
-- Design      : reconfiguration
-- Author      : Vegard
-- Description : Shift register with full paralell-serial interface. Parameterized
--				 input / output size
-------------------------------------------------------------------------------

library IEEE;
use IEEE.STD_LOGIC_1164.all;

entity shift_register_PIOSIZE is
	generic(
		--register size
		C_SIZE			: natural;
		C_INPUT_SIZE 	: natural	
	);

	port(
				
		clk			: in  std_logic;  
		par_i   	: in  std_logic_vector(C_INPUT_SIZE-1 downto 0);
		par_write	: in  std_logic;
		shift		: in  std_logic;
		shift_i		: in  std_logic;
		shift_o		: out std_logic;
		par_o		: out std_logic_vector(C_SIZE-1 downto 0)
	 );
	

end entity shift_register_PIOSIZE;
					  
architecture arch of shift_register_PIOSIZE is 

signal memory 	: std_logic_vector(C_SIZE-1 downto 0);

begin
	
	par_o 	<= memory;
	
	WRITE_PROC : process(clk) is
	begin
	if(clk'event and clk ='1') then
		if shift ='1' then
			for i in C_SIZE-2 downto 0 loop
				memory(i) <= memory(i+1);
			end loop;
			memory(C_SIZE-1) <= shift_i;
		elsif par_write ='1' then
			memory(C_SIZE-1 downto (C_SIZE-C_INPUT_SIZE) ) <= par_i; 
		else	
			memory <= memory;	
		end if;
	end if;
	end process WRITE_PROC; 
								
	SHIFT_OUT_PROC: process(clk) is
	begin
	if(clk'event and clk='0') then
		shift_o <= memory(0);
	end if;
	end process SHIFT_OUT_PROC;
	
	
end architecture arch;
