-------------------------------------------------------------------------------
--
-- Title       : shift_chain
-- Design      : reconfiguration
-- Author      : Vegard Endresen
-- Company     : NTNU
-- Description : State machine template with state integrated in scan chain
-------------------------------------------------------------------------------

library IEEE;
use IEEE.STD_LOGIC_1164.all;

entity sm_serial is
	port(
		clk			: in  std_logic; 
		run			: in  std_logic;
		shift		: in  std_logic;
		shift_i		: in  std_logic;
		shift_o		: out std_logic;
		HW_complete	: out std_logic
	);

end sm_serial;


architecture arch of sm_serial is

constant C_STATE_SIZE : natural :=4;

------------------------------------------------------------------
-- SM REG 

signal nstate 			: std_logic_vector(C_STATE_SIZE-1 downto 0);
signal cstate 			: std_logic_vector(C_STATE_SIZE-1 downto 0);
signal state_write 		: std_logic; 
--------------------------------------------------------------------

--------------------------------------------------------------------
-- STATE CONSTANTS
constant s0		: std_logic_vector(C_STATE_SIZE-1 downto 0) := x"0";
constant s1		: std_logic_vector(C_STATE_SIZE-1 downto 0) := x"1";
constant s2		: std_logic_vector(C_STATE_SIZE-1 downto 0) := x"2";
constant s3		: std_logic_vector(C_STATE_SIZE-1 downto 0) := x"3";
constant s4		: std_logic_vector(C_STATE_SIZE-1 downto 0) := x"4";
constant s5		: std_logic_vector(C_STATE_SIZE-1 downto 0) := x"5";
constant s6		: std_logic_vector(C_STATE_SIZE-1 downto 0) := x"6";
constant s7		: std_logic_vector(C_STATE_SIZE-1 downto 0) := x"7";
constant s8		: std_logic_vector(C_STATE_SIZE-1 downto 0) := x"8";
constant s9		: std_logic_vector(C_STATE_SIZE-1 downto 0) := x"9";
constant s10	: std_logic_vector(C_STATE_SIZE-1 downto 0) := x"A";
constant s11	: std_logic_vector(C_STATE_SIZE-1 downto 0) := x"B";
constant s12	: std_logic_vector(C_STATE_SIZE-1 downto 0) := x"C";
constant s13	: std_logic_vector(C_STATE_SIZE-1 downto 0) := x"D";
constant s14	: std_logic_vector(C_STATE_SIZE-1 downto 0) := x"E";
constant s15	: std_logic_vector(C_STATE_SIZE-1 downto 0) := x"F";
--------------------------------------------------------------------


begin

	
--------------------------------------------------------------------
-- SHIFT CHAIN 

	STATE_REG : entity shift_register
    	generic map(C_SIZE => C_STATE_SIZE) 
		port map
	    (	clk			=> clk,
			par_i 		=> nstate,
			par_write 	=> state_write,
			shift 		=> shift,
			shift_i 	=> shift_i,
			shift_o 	=> shift_o,
			par_o		=> cstate
	    );			
--------------------------------------------------------------------	
	
	WRITE_PROC : process(run,cstate) is
	begin
		if run ='0' then
			state_write		<='0';
			nstate			<=s0;
		else   
			state_write 	<='1';
			--------------------------------------------
			-- state machine code
			case cstate is
				when s0 =>
					nstate <= s1;
				when s1 =>
					nstate <= s2;
				when s2 =>
					nstate <= s3;
				when s3 =>
					nstate <= s4;
				when s4 => 
					nstate <= s5;
				when s5 =>
					nstate <= s6;
				when s6 =>
					nstate <= s7;
				when s7 =>
					nstate <= s8;
				when s8 =>
					nstate <= s9;
				when s9 =>
					nstate <= s10;
				when s10 =>
					nstate <= s11;
				when s11 =>
					nstate <= s12;
				when s12 =>
					nstate <= s13;
				when s13 =>
					nstate <= s14;
				when s14 =>
					nstate <= s15;
				when s15 =>
					nstate <= s0;
				when others =>
				
			end case;
			--------------------------------------------
		end if;
		
	end process;
	
	HW_COMPLETE_PROC : process(cstate) is
	begin
		if cstate = s15 then
			HW_complete <= '1';
		else
			HW_complete <='0';
		end if;	

	end process HW_COMPLETE_PROC;
	
	
end arch;
