-------------------------------------------------------------------------------
--
-- Title       : testbench
-- Design      : serial_controller
-- Author      : Vegard	Endresen
-- Company     : NTNU
-- Description : Testbench for shift and serial controller. DUT uncommented
-------------------------------------------------------------------------------

library IEEE;
use IEEE.STD_LOGIC_1164.all;

entity testbench is
end testbench;


architecture TEST of testbench is

---------------------------------------------------------
-- Common fields

constant T_CLK 			: time	  := 10 ns;
constant C_COUNT_SIZE 	: natural := 4;

signal clk	 : std_logic; 
signal shift : std_logic;
signal start : std_logic; 
signal count : std_logic_vector(C_COUNT_SIZE-1 downto 0);
---------------------------------------------------------


---------------------------------------------------------
-- Shift controller fields

signal shift_complete : std_logic;
---------------------------------------------------------


---------------------------------------------------------
-- Serial controller fields

signal run : std_logic;
signal CPU_shift : std_logic;
signal CPU_run : std_logic;
signal CPU_interrupt : std_logic;
signal HW_complete : std_logic;
signal controller_waiting : std_logic;
---------------------------------------------------------

begin
	
	-- Clock generation
	CLKGEN: process is
	begin
		clk <= '1';
		wait for T_CLK/2;
		clk <= '0';
		wait for T_CLK/2;
	end process;

---------------------------------------------------------
-- Shift controller test

	SHIFT_CONTROLLER: entity controller
	generic map(C_COUNT_SIZE => C_COUNT_SIZE)
	port map(
		clk => clk,	
		count => count,
    	start => start,
		shift => shift,
		shift_complete => shift_complete
	);
	
	STIMULI : process is
	begin		  
		-- Generate shift for 5 edges
		count <=x"4";
		start <='0';
		wait for T_CLK+T_CLK/2;
		start <='1';
		wait for T_CLK;
		start <='0';
		wait;
		
	end process STIMULI;
---------------------------------------------------------	


-----------------------------------------------------------
----Serial controller test
--
--
--
--	SERIAL_CONTROLLER : entity serial_controller
--	generic map(C_COUNT_SIZE => C_COUNT_SIZE)
--	port map(
--		clk 			=> clk,	 
--		CPU_interrupt	=> CPU_interrupt,
--		CPU_shift 		=> CPU_shift,
--		CPU_run 		=> CPU_run,
--		count 			=> count,
--		shift			=> shift,
--		HW_complete 	=> HW_complete,
--		run 			=> run,
--		controller_waiting => controller_waiting
--		); 
--	
--		
--	STIMULI : process is
--	begin		  
--	
--		count 	<=x"A";
--		HW_complete <='0';
--		
--		CPU_interrupt <='0'; 
--		CPU_shift 	  <='0';
--		CPU_run 	  <='0';
--		
--		wait for T_CLK;	 
--		
--		CPU_interrupt <='0'; 
--		CPU_shift 	  <='1';
--		CPU_run 	  <='0';
--		
--		while controller_waiting ='0' loop
--			wait for T_CLK;
--		end loop;
--		
--		CPU_interrupt <='1'; 
--		CPU_shift 	  <='0';
--		CPU_run 	  <='0';
--		
--		wait for T_CLK;
--		
--		count 	<=x"2";
--		CPU_interrupt <='0'; 
--		CPU_shift 	  <='1';
--		CPU_run 	  <='0';
--		
--		while controller_waiting ='0' loop
--			wait for T_CLK;
--		end loop;
--		
--		CPU_interrupt <='1'; 
--		CPU_shift 	  <='0';
--		CPU_run 	  <='0'; 
--		
--		wait for T_CLK;
--		
--		CPU_interrupt <='0'; 
--		CPU_shift 	  <='0';
--		CPU_run 	  <='1'; 
--		
--		wait for T_CLK*5;
--		
--		HW_complete <='1';
--		
--		wait;
--	end process STIMULI;	
-----------------------------------------------------------

end TEST;
