/*************************************************************
*File			: multiplier.c
*Author			: Vegard Endresen
*Description	: Client application using device driver to access
*				  data cache backend.
**************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <stdint.h>
#include <fcntl.h>
#define WBUFSIZE 20
#define RBUFSIZE 4
#define IBUFSIZE 120



#define BACKEND "/var/d_backend" //Device file path

/*Character array vs integer conversion functions*/

static void intToChar(char* buf, int value, int pos){
	int i,mask,tmp;
	mask = 0x000000FF;	
	for(i=0; i< 4; i++){
		tmp = (value & mask) >>(8*i);
		if(i==3) tmp = tmp & 0x000000FF;		;		
		buf[i+pos] =tmp;				
		mask = (mask <<8);
	}

}

static int charToInt(char *buf, int pos){
	int res,i,mask;
	mask = 0x000000FF;	
	res =0;
	for(i=0; i< 4; i++){
		res = res | ((buf[pos+i]<< 8*i) & mask);
		mask = (mask <<8);	
	}
	return res;
}

/*Perform instruction write and wait for completion*/
void synchronized_write(int handlemem,char *buf){
	int timer =0;	
	int ret;
	ret = pwrite(handlemem,buf,4, 1024);
	while(1==1){
		 ret= pread(handlemem,buf,4,1024);
		if((charToInt(buf,0) & 00000001) ==1){
			break;		
		}
		timer++;
		if(timer> 1000){
			printf("timer exipred\n");
			break;
		}
	}
	intToChar(buf,0x10004000,0);
	ret = pwrite(handlemem,buf,4, 1024);
	return;
}

int main(int argc, char * argv[])
{
  	printf("Application: multiplier\n");
	
	char *writebuf = malloc(sizeof(char)*WBUFSIZE);
	char *readbuf = malloc(sizeof(char)*RBUFSIZE);
	char *instrbuf = malloc(sizeof(char)*IBUFSIZE);

	int cnt; 
	int handlemem;
	int ret,i,addr,data,instr;

	printf("Initializing buffers ...\n");
	for(i=0; i< WBUFSIZE; i++){
	  writebuf[i] = 0;	
	}
	
	for(i=0; i< RBUFSIZE; i++){
	  readbuf[i] = 0;	
	}

	for(i=0; i< IBUFSIZE; i++){
	  instrbuf[i] = 0;	
	}

	printf("Device driver open ...\n");
	handlemem = open(BACKEND,O_RDWR);
	if(handlemem == -1){
		printf("Device driver error\n");
		goto err;
	}
	else printf("Decive driver accessed HW\n");
	
	printf("Initializing BRAM ...\n");
	ret = pwrite(handlemem, writebuf,WBUFSIZE, 0);
	
	intToChar(writebuf,0xF000F000,0);
	printf("Writing sync OP ...\n");
	ret = pwrite(handlemem, writebuf,4, 0);		
	
	printf("Writing data to user space buffer ...\n");

	if(WBUFSIZE < 20){
		printf("bad generics\n");
		return 0;	
	}	

	intToChar(writebuf,0xA2000000,0);
	intToChar(writebuf,0x5BF134F0,4);
	intToChar(writebuf,0xABCDEF12,8);
	intToChar(writebuf,0x00ABCDEF,12);
	intToChar(writebuf,0x0000004C,16);
	
	//transfer entire buffer in one go
	printf("Transferring data to BRAM...\n");	
	ret = pwrite(handlemem, writebuf,WBUFSIZE,0);
	
	printf("Resetting timer ...\n");
	intToChar(readbuf,0x00008000,0);
	ret = pwrite(handlemem, readbuf,4,1024);

	printf("Building instruction buffer ...\n");
	cnt=0;
	for(i= 0; i<4; i++ ){
		addr  = i << 16;		
		instr = 0x20004000 | addr;	
		intToChar(instrbuf,instr,cnt);	
		intToChar(instrbuf,0xA0034000,(cnt+4));		
		if(i !=3)cnt +=8;
		else cnt +=4;
	}
	intToChar(instrbuf,0x20044000,(cnt+=4));
	intToChar(instrbuf,0xA0044000,(cnt+=4));
	intToChar(instrbuf,0xE0000000,(cnt+=4));

	//destination start address
	addr = 8;
	for(i=0; i< 2; i++){
		intToChar(instrbuf,0xA0030000,(cnt+=4));
		instr = 0x60000000 | addr << 16;		
		intToChar(instrbuf,instr,(cnt+=4));
		addr = addr+1;
	}


	for(i=0; i< 30; i++){
		printf("buf[%d]: %x\n",i,charToInt(instrbuf,i*4));
	} 

	intToChar(readbuf,0x10000000,0);
	ret = pwrite(handlemem, readbuf,4, 1024);
	printf("Running program ...\n");	
	ret = pwrite(handlemem, writebuf,WBUFSIZE,0);
	for(i=0; i< 10; i++){
		data = charToInt(instrbuf,i*4);		
		intToChar(readbuf,data,0);
		synchronized_write(handlemem,readbuf);
	}
	intToChar(readbuf,0x10000000,0);
	ret = pwrite(handlemem, readbuf,4, 1024);

	ret = pread(handlemem, readbuf,4, 1028);
	printf("count : %x\n",charToInt(readbuf,0));
	
	/*int timer =0;
	while(1==1){
		ret = pread(handlemem, readbuf,4, 1024); 		
		if((charToInt(readbuf,0) & 00000001) ==1){ 			
			intToChar(readbuf,0x00000000,0);	
			ret = pwrite(handlemem, readbuf,4, 1024);
			ret = pread(handlemem, readbuf,4, 1024);
			printf("Reg %x\n", charToInt(readbuf,0));						
			
			ret = pread(handlemem, readbuf,4, 1028);
			printf("Stopped counter at %x\n", charToInt(readbuf,0));						
			break;
		}	
	}

	for(i=20; i< 30; i++){
		data = charToInt(instrbuf,i*4);		
		intToChar(readbuf,data,0);
		ret = pwrite(handlemem, readbuf,4, 1024);
		ret = pread(handlemem, readbuf,4, 1024);
		data = charToInt(readbuf,0);		
		printf("reg: %x\n",data);
	}

	
	for(i=0; i< 10; i++){
		ret = pread(handlemem,readbuf,4, i*4);
		if(ret!=4) cnt++; 
		printf("BRAM[%d]: %x\n",i,charToInt(readbuf,0));
	
	}
	ret = pread(handlemem,readbuf,4, 1024);
	printf("REG : %x\n",charToInt(readbuf,0));*/
	err:
	  	close(handlemem);
	 	free(writebuf);
		free(readbuf);
		free(instrbuf);
  return 0;
}
