/*************************************************************
*File			: multiplier.c
*Author			: Vegard Endresen
*Description	: Client application accessing hardware through
*				  the prototype HWOS
**************************************************************/
#include<stdio.h>
#include<stdlib.h>
#include <sys/ipc.h> 
#include <sys/msg.h>
#include <string.h>
#include "hwos_if.h"

key_t key; /* key to be passed to msgget() */ 
int msgflg; /* msgflg to be passed to msgget() */ 
int msqid; /* return value from msgget() */ 

key_t recv_key;
int recv_id;


/*Character array vs integer conversion functions*/

static void intToChar(char buf[], int value, int pos){
	int i,mask,tmp;
	mask = 0x000000FF;	
	for(i=0; i< 4; i++){
		tmp = (value & mask) >>(8*i);
		if(i==3) tmp = tmp & 0x000000FF;		;		
		buf[i+pos] =tmp;				
		mask = (mask <<8);
	}

}

static int charToInt(char buf[], int pos){
	int res,i,mask;
	mask = 0x000000FF;	
	res =0;
	for(i=0; i< 4; i++){
		res = res | ((buf[pos+i]<< 8*i) & mask);
		mask = (mask <<8);	
	}
	return res;
}

int main(int argc, char **argv){
	int i,ret,base,id,instr;
	req_msg_t *msgp;
	resp_msg_t *recv;

	/*Connect to HWOS primary message queue*/
	key = 1234;
	msgflg= IPC_CREAT | 0666;

	if ((msqid = msgget(key, msgflg)) == -1){
		printf("Primary queue failure\n");
    	return -1;
	} 
	else printf("Connected to primary\n");


	/*Create receive queue*/
	recv_key = 4321;
	if ((recv_id = msgget(recv_key, msgflg)) == -1){
		printf("Receive queue failure\n");
    	return -1;
	} 
	else printf("message queue created\n");

	
	msgp = malloc(sizeof(req_msg_t));
	recv = malloc(sizeof(resp_msg_t));


	/*Register message queue*/
	msgp->sender_id = recv_key;
	msgp->type = 12;	
	msgp->bytes = 18*4;
	msgp->command =CMD_RMQUE;
	msgp->addr=0; 
	intToChar(msgp->data,recv_id,0);
	ret = msgsnd(msqid,msgp, sizeof(req_msg_t),0);
	ret = msgrcv(recv_id,recv,sizeof(resp_msg_t), 11,0);
	printf("register: %d\n", recv->retval);
	
	/*msgp->command = CMD_LDDEV;
	intToChar(msgp->data,128,0);
	ret = msgsnd(msqid,msgp, sizeof(req_msg_t),0);
	ret = msgrcv(recv_id, recv, sizeof(resp_msg_t), 11,0);*/

	/*Allocate 18 lines of BRAM memory*/
	msgp->command=CMD_ALLOC;
	intToChar(msgp->data,7,0);	
	ret = msgsnd(msqid,msgp, sizeof(req_msg_t),0);
	ret = msgrcv(recv_id, recv, sizeof(resp_msg_t), 11,0);
	if(recv->buf == -1) {
		printf("allocation failed\n");	
		return -1; 
	}
	
	printf("alloc : %d %d\n",recv->retval,recv->buf);

	
	

	/*Allocation id assigned by HWOS*/	
	id = recv->retval;
	/*Base line-address assigned by HWOS*/
	base = recv->buf;	

	/*Initialize instruction segment*/
	intToChar(msgp->data,0xF000F000,0); //HALT	
	for(i=1; i< 22; i++){
		intToChar(msgp->data,0,i*4);
	}

	msgp->bytes = 22*4;
	msgp->addr = 0;
	msgp->command=CMD_WRITE;
		
	ret = msgsnd(msqid,msgp, sizeof(req_msg_t),0);
	ret = msgrcv(recv_id,recv,sizeof(resp_msg_t), 11,0);
	printf("write: %d\n", recv->retval);

	/*Write data to assigned memory*/
	intToChar(msgp->data,0xA2000000,0);
	intToChar(msgp->data,0x5BF134F0,4); 
	intToChar(msgp->data,0xABCDEF12,8);
	intToChar(msgp->data,0x00ABCDEF,12);
	intToChar(msgp->data,0x0000004C,16);
	intToChar(msgp->data,0xACDCACDC,20);
	intToChar(msgp->data,0xBCDCBDCD,24);

	msgp->bytes = 7*4;
	msgp->addr = base*4;
	msgp->command=CMD_WRITE;
		
	ret = msgsnd(msqid,msgp, sizeof(req_msg_t),0);
	ret = msgrcv(recv_id,recv,sizeof(resp_msg_t), 11,0);
	
	printf("data write %d\n", recv->retval);

	/*Reset timer*/
    intToChar(msgp->data,0x8F000000,0);
	msgp->bytes = 4;
	msgp->addr = 1024;
	msgp->command=CMD_WRITE;
	ret = msgsnd(msqid,msgp, sizeof(req_msg_t),0);
	ret = msgrcv(recv_id,recv,sizeof(resp_msg_t), 11,0);

	
	/*Start timer*/
	intToChar(msgp->data,0x4F000000,0);
	msgp->bytes = 4;
	msgp->addr = 1024;
	msgp->command=CMD_WRITE;
	ret = msgsnd(msqid,msgp, sizeof(req_msg_t),0);
	ret = msgrcv(recv_id,recv,sizeof(resp_msg_t), 11,0);

	
	intToChar(msgp->data,0xF000F000,0);
	/*Compile program with respect to base address*/
	for(i=1; i< 5; i++){
			intToChar(msgp->data,0x80030000 |(base+i),i*4);
	}
	instr = 0xF000F000;
	intToChar(msgp->data,instr,4*5);
	/*instr = 0x0000C000 | ((base+5)<<16);
	printf("i: %x\n",instr);	
	intToChar(msgp->data,instr,20);
	instr = 0x40008013 | ((base+7)<<16);
	intToChar(msgp->data,instr,24);
	instr = 0x80130000 | (base+6);
	intToChar(msgp->data,instr,28);
	instr = 0xC0004000 | (base+8); 
	intToChar(msgp->data,instr,32);
	instr = 0x40008013 | ((base+9)<<16);	
	intToChar(msgp->data,instr,36);
	instr = 0x40008013 | ((base+10)<<16);
	intToChar(msgp->data,instr,40);*/

	msgp->bytes = 24;
	msgp->command=CMD_EXEC;

	ret = msgsnd(msqid,msgp, sizeof(req_msg_t),0);
	ret = msgrcv(recv_id,recv,sizeof(resp_msg_t), 11,0);

		
	/*msgp->bytes = 4;
	msgp->addr = 0;
	msgp->command=CMD_READ;		
	for(i=0; i< 20; i++){
		ret = msgsnd(msqid,msgp, sizeof(req_msg_t),0);
		ret = msgrcv(recv_id, recv, sizeof(resp_msg_t), 11,0);	
		printf("reg %x\n",charToInt(recv->data,0));		
		msgp->addr = msgp->addr+4;
	}*/
	



	/*Execute*/
	intToChar(msgp->data,0x80030000 |base,0);		
	msgp->bytes = 4;
	msgp->command=CMD_EXEC;
	ret = msgsnd(msqid,msgp, sizeof(req_msg_t),0);
	ret = msgrcv(recv_id,recv,sizeof(resp_msg_t), 11,0);
	

	
	msgp->bytes = 4;
	msgp->addr = 1024; 
	msgp->command=CMD_READ;

	i=0;	
	while(1){
		ret = msgsnd(msqid,msgp, sizeof(req_msg_t),0);
		ret = msgrcv(recv_id, recv, sizeof(resp_msg_t), 11,0);	
		if( (charToInt(recv->data,0) & 0x00000001) == 1){
			intToChar(msgp->data,0x0F000000,0);
			msgp->bytes = 4;
			msgp->addr = 1024;
			msgp->command=CMD_WRITE;		
			ret = msgsnd(msqid,msgp, sizeof(req_msg_t),0);
			ret = msgrcv(recv_id, recv, sizeof(resp_msg_t), 11,0);	
			break;
		}
	}
	
	msgp->bytes = 4;
	msgp->addr = 1028; 
	msgp->command=CMD_READ;
	ret = msgsnd(msqid,msgp, sizeof(req_msg_t),0);
	ret = msgrcv(recv_id, recv, sizeof(resp_msg_t), 11,0);	
	printf("timer at %x\n", charToInt(recv->data,0));	



	msgp->command=CMD_FREE;
	intToChar(msgp->data,id,0);	
	ret = msgsnd(msqid,msgp, sizeof(req_msg_t),0);
	ret = msgrcv(recv_id,recv,sizeof(resp_msg_t), 11,0);
	printf("free: %d\n", recv->retval);	
	
	free(msgp);
	free(recv);
	return 0;
}

