/********************************************************
*File			: hwos_if.h
*Author			: Vegard Endresen
*Description	: Header definition for HWOS interface.
*********************************************************/
#ifndef HWOS_H
#define HWOS_H

/*Message types, little parctical influence*/
#define BACKEND_RD 0; /*Client->HWOS*/
#define BACKEND_WR 1; /*HWOS->Client*/

#define BUF_SIZE 128


#define MSG_CMD 0
#define MSG_RET 1

/*Device file root folder on Suzaku*/
#define DEVICE_ROOT "/var/tmp/dev"

/*
*Command map, commands that can be sent
*through messages from a client applicatoin
*/
#define CMD_ALLOC 	 0 //Allocate BRAM memory
#define CMD_EXEC	 1 //Write program to instruction segment of BRAM
#define CMD_FREE	 2 //Free BRAM memory
#define CMD_LDDEV	 3 //Load device driver
#define CMD_READ  	 4 //Read data from backend
#define CMD_RMDEV	 5 //Remove device driver
#define CMD_RMQUE 	 6 //Register receive message queue to hwos
#define CMD_UMQUE 	 7 //Unregister receive message queue to hwos
#define CMD_WRITE 	 8 //Write data to backend



/*REQ_MSG, Client -> HWOS*/
typedef struct req_msg{
	long 	type;			//This field must be first in struct,else errors will appear when sending
	char 	command;		//Temporary command field(see command legend).
	int     addr;			//Operation base_address
	int 	bytes;			//Number of lines to be read/written
	char 	data[BUF_SIZE]; //Data buffer,cannot use pointers by default because of separated memory spaces
	int 	sender_id;		//Senders message queue id for receiving response
} req_msg_t;

/*RESP_MSG, HWOS -> Client*/
typedef struct resp_msg{
	long 	type;	
	int		retval;
	int		buf;
	char 	data[BUF_SIZE];
} resp_msg_t;



#endif
