/*************************************************************
*File			: devicelist.c
*Author			: Vegard Endresen
*Description	: Descriptor list of all the loaded device drivers
**************************************************************/
#include<stdio.h>
#include<stdlib.h>
#include <string.h>
#include "devicelist.h"

/*
*FUNCTIONS
*/
int  getNumDevices();
int  getDeviceByMajor(int major, device_t **search_pointer);
void insertDevice(int major, char devicePath[],int pathSize);
void removeDevice(int major);
void clearDevices();
device_t * getFirstDevice();



//number of elements in structure
int dev_cnt =0;
//first element in pool list
device_t *firstd,*lastd;

int getNumDevices(){
	return dev_cnt;
}

/*
*Copy memory pool descriptor to argument
*/
int getDeviceByMajor(int major,device_t **dev){
	if(dev_cnt==0) return -1;
	*dev = firstd;	
	if((*dev)->major == major) return 0;
	while(1){
		//last item
		if((*dev)->next ==0) return -1;	   

		*dev = (*dev)->next;
		if((*dev)-> major == major) return 0;	
	}	

}

/*
*Insert pool at end of list
*/
void insertDevice(int major,char devicePath[], int pathSize){
	int ret;
	device_t *dev = malloc(sizeof(device_t));
	device_t **search_pointer = malloc(sizeof(int));
	
	ret = getDeviceByMajor(major,search_pointer);	
	free(search_pointer);
	if(ret != -1) goto err;
	
	dev->major = major;
	//NB max path size is 40 characters
	memcpy(dev->devicePath,devicePath,pathSize);	

	//first
	if(dev_cnt ==0){
		firstd = dev;
		lastd  = dev; 
		dev->prev =0;
	}	
	else{
		lastd->next = dev;
		dev->prev = lastd;
		lastd = dev;	
	}
	dev_cnt++;
	return;
	
	err:
		printf("err\n");
		free(dev);

}


/*
*Removes pool element where pool->id == id 
*/
void removeDevice(int major){	
	int ret;
	device_t **search_pointer;
	device_t *dev;

	search_pointer = malloc(sizeof(int));
	ret = getDeviceByMajor(major,search_pointer);
	if(ret == -1) return;
	dev = *search_pointer;
	free(search_pointer);

	//only element left
	if(dev_cnt ==1){
		free(firstd);
		lastd =0;
		dev_cnt --;
		return;
	}
	//first element
	if(dev->prev ==0){
		dev  = dev->next;
		firstd = dev;
		free(dev->prev);
		dev->prev=0;			
	}
	//last element
	else if(dev->next ==0){
		dev  = dev->prev;
		lastd = dev;
		free(dev->next);
		dev->next=0;	
	}
	//in-between
	else{
		dev->prev->next = dev->next;
		dev = dev->next;
		device_t *tmp = dev->prev;
		dev->prev = dev->prev->prev;
		free(tmp);
	}

	dev_cnt--;
}

void clearDevices(){
	device_t *it = firstd;
	while(it->next !=0){
		it = it->next;
		removeDevice(it->prev->major);
	}
	removeDevice(it->major);
}

device_t *getFirstDevice(){
	return firstd; 
}

/*Testing*/
/*
void printDeviceList(){
	device_t *iterator = getFirstDevice();
	printf("#########################\n");		
	while(1){	
		printf("%d\t\t %s \n",iterator->major,iterator->devicePath);		
		if(iterator-> prev != 0) printf("prev : %d\n", iterator->prev->major);		
		if(iterator->next !=0){
			printf("next  : %d\n", iterator->next->major);					
			iterator = iterator->next;	
		}
		else break;	
	}
	printf("#########################\n");
}


int main(int argv,char ** argc){

	insertDevice(20,"/dev/backend1.ko",17);
	insertDevice(41,"/tmp/backend2.ko",17);
	insertDevice(54,"/dev/other.ko",14);
	insertDevice(31,"/dev/a.ko",10);
	printf("cnt %d\n", dev_cnt);
	printDeviceList();
	
	return 0;
}
*/


