/*************************************************************
*File			: dmem_kernel.c
*Author			: Vegard Endresen
*Description	: Dynamic BRAM memory allocation implementation. 
*				  The underlying structure should support more	
*				  advanced expansion.
*Bug			: If line 65 fails, cleanup() crashes
**************************************************************/

#include<stdio.h>
#include<stdlib.h>
#include <pthread.h>
#include "memory.h"
#include "pools.h"
#define INSTR_BUF_SIZE 0xA
#define POOL_SIZE 16

#define BACKEND "/var/id_backend" 

/*
*FUNCTIONS
*/
int getId();
int allocate(int size,int *addr);
void freeMem(int id);


int instrWrite(int id,char *buf,int size);
int dataWrite(int id,char *buf,int addr,int size);
int dataRead(int id, char *buf,int addr,int size);

void printDescriptors();
void dmemInit(int bram_size);
void dmemCleanUp();

int data_base;
int size_;
//id map for memory allocation
int *id_map;

//mutual exclusion for allocate() and free_mem() operations
pthread_mutex_t mp; 

int handlemem;

/*
*Default init of pool list divides
*BRAM into uniform chunks of POOL_SIZE lines.  
*Ownership is marked with -1 for NONE.
*Id_map is initialized.	
*/
void dmemInit(int bram_size){
	int i;	
	printf("init...\n");
	//compensate for instruction segment
	data_base = ((INSTR_BUF_SIZE/POOL_SIZE)+1);
	
	for(i=data_base; i<bram_size/POOL_SIZE; i++){
		insertPool(-1,POOL_SIZE*i,POOL_SIZE);
	}

	id_map = malloc(sizeof(int)*10);	
	for(i=0; i< 10; i++){
		id_map[i] =-1;
	}
	
	printf("Opening device driver\n");
	/*handlemem = open(BACKEND,O_RDWR);
	if(handlemem == -1){
		printf("Device driver error\n");
		goto err;
	}
	else printf("Decive driver accessed HW\n");
	*/
	printf("init complete...\n");
}

void dmemCleanUp(){
	printf("cleaning up...\n");	
	//free id_map	
	free(id_map);
	//free descriptor list
	clearAll();
	printf("cleanup complete...\n");
}

/*
*Returns a unique id, -1 if an id cannot be obtained
*/
int getId(){
	int i;	
	for(i=0; i<10; i++){
		if(id_map[i] ==-1){
			id_map[i]=i;
			return i;
		}
	}
	return -1;
}

/*
*Memory allocation algorithm.
*Iterate through the BRAM pool descriptor
*and find the first continous set 
*of pools that is large enough to fit $size 
*/
int allocate(int size,int *addr){
	int space,base_addr,ret,id;	
	pool_t *iterator;
	pool_t **search_pointer;
	ret = pthread_mutex_lock(&mp);	
	
	search_pointer = malloc(sizeof(int));
	if(size ==0 || (id = getId()) == -1) goto err;
				
	space =0;

	ret = getPoolByAddr(data_base*POOL_SIZE,search_pointer);
	if(ret ==-1) goto err;	
	printf("there\n");	
	iterator = search_pointer[0];
	base_addr = iterator->base_addr;
	
	while(1){
		
		if(base_addr==-1) base_addr = iterator->base_addr;
		
		if(iterator->id ==-1) space += iterator->size;	
		else{
			space=0;
			base_addr =-1;
		}		
	
		if(space > size) break;		
		
		if(iterator->next !=0) iterator = iterator->next;	
		else break;	
	}	
	if(space< size) goto err;

	//found space, mark descriptors as taken 
	ret = getPoolByAddr(base_addr,search_pointer);
	
	if(ret ==-1) goto err;	
	iterator = search_pointer[0];
	
	space =0;
	while(space< size){
		space += iterator->size;
		iterator->id = id;
		if(iterator->next !=0) iterator = iterator->next;	
		else break;	
	}
	
	*addr = base_addr;
	iterator =0;	
	free(search_pointer);	
	pthread_mutex_unlock(&mp);
	return id;

	err:
		iterator =0;
		free(search_pointer);	
		pthread_mutex_unlock(&mp);
		return -1;
}

/*
*Frees memory previously allocated by id.
*Will also free id in the id_map. 
*/
void freeMem(int id){
	int ret;
	pool_t *iterator;
	pool_t **search_pointer;
	
	ret = pthread_mutex_lock(&mp);

	search_pointer = malloc(sizeof(int));
	ret = getPoolById(id,search_pointer);
	if(ret ==-1) return; //bad
	iterator = search_pointer[0];

	while(1){
		if(iterator->id != id) break;
		else iterator->id = -1;
		
		if(iterator->next !=0) iterator = iterator->next;	
		else break;			
	}		
	iterator =0;
	free(search_pointer);
	id_map[id] =-1;
	pthread_mutex_unlock(&mp);
}

int instrWrite(int id,char *buf,int size){
	return 0;	
	//return  pwrite(handlemem,buf,size,0);
}
int dataWrite(int id,char *buf,int addr,int size){
	/*pool_t **search_pointer;
	pool_t *iterator;
	int i,ret,cnt;
	char *transfer_buf;	
	cnt =0;
	*/
	return 0;	
	//return = pwrite(handlemem,buf,size,addr);

	/*Memory Protection, disregarded for now*/
	/*
	search_pointer = malloc(sizeof(int));
	ret = getPoolByAddr(addr,search_pointer);	
	if(ret ==-1) goto err;
	iterator = search_pointer[0];
	while(1){
		//check if caller owns pool
		if(iterator->id != id) goto err;
		int taddr = addr%iterator->size;
		int tsize;		
		if((size-cnt) > iterator->size) tsize=iterator->size;
		else tsize= size-cnt;

		char *tbuf= malloc(sizeof(int)*tsize);
		for(i=0; i< tsize*sizeof(int); i++){
			tbuf[i] = buf[]

		}	

		
		cnt+=tsize;		
		printf("cnt %d in  pool %d\n",cnt,iterator->id);		
		if(cnt ==size){
			goto out;	
		}		
		if(iterator->next ==0)	goto err;
		iterator = iterator->next;
	}

	out:
		iterator =0;
		free(search_pointer);	
	
	return ret;

	err:
		printf("err\n");
		iterator =0;
		free(search_pointer);
		return -1;	*/
}
int dataRead(int id,char *buf,int addr,int size){
	/*pool_t **search_pointer;
	pool_t *iterator;
	int i,ret,cnt;
	cnt =0;*/
	return 0;
	//ret = pwrite(handlemem, buf,size,addr);

	/*Memory Protection, disregarded for now*/
	/*
	search_pointer = malloc(sizeof(int));
	ret = getPoolById(id,search_pointer);	
	if(ret ==-1) goto err;
	iterator = search_pointer[0];
	while(cnt < size){
		//check if caller owns pool
		if(iterator->id != id) goto err;
		int taddr = addr%iterator->size;
		if((size-cnt) > iterator->size) cnt+=iterator->size;
		else cnt+= size-cnt;
		
		printf("cnt %d in  pool %d\n",cnt,iterator->id);		
		if(cnt ==size){
			goto out;	
		}		
		if(iterator->next ==0)	goto err;
		iterator = iterator->next;
	}
	out:
		iterator =0;
		free(search_pointer);	

	return 0;

	err:
		iterator =0;
		free(search_pointer);
		return -1;	*/
	return 0;
}



/*
*TESTING
*/

void printDescriptors(){
	pool_t *iterator = getFirst();
	printf("#########################\n");	
	printf("Addr\t\t Id\n");	
	while(1){	
		printf("%d\t\t %d\n",iterator->base_addr,iterator->id);		
		if(iterator->next !=0) iterator = iterator->next;	
		else break;	
	}
	printf("#########################\n");
}



/*int main(){
	
	int i;	
	//int cnt;	
	int **addr;	
	int *ret;
	
	size_ = 256;	
	dmemInit(size_);
	printDescriptors();
	
	addr = malloc(sizeof(int)*3);
	ret  = malloc(sizeof(int)*3);
	for(i=0; i<3; i++){
		addr[i] = malloc(sizeof(int));
	}	
	
	//allocate
	ret[0] = allocate(17,addr[0]);
	ret[1] = allocate(67,addr[1]);
	ret[2] = allocate(40,addr[2]);

	for(i=0; i<3; i++){
		printf("ret %d\n",ret[i]);
	}		
	
	printDescriptors();
	//free	
	free_mem(ret[1]);

	printDescriptors();
	
	//allocate
	ret[1] = allocate(12,addr[1]);
	
	printDescriptors();

	//free pointers
	for(i=0; i<3; i++){
		free(addr[i]);
	}	
	free(addr);
	free(ret);
	
	dmemCleanUp();


	return 0;
}*/

