/********************************************************
*File			: hwos_if.h
*Author			: Vegard Endresen
*Description	: Top level DEAMON skelleton 
*********************************************************/
#include <sys/types.h>
#include <sys/stat.h>
#include <stdio.h>
#include <stdlib.h>
#include <fcntl.h>
#include <errno.h>
#include <unistd.h>
#include <syslog.h>
#include <string.h>
#include <signal.h>

/*Message queue*/
#include <sys/ipc.h> 
#include <sys/msg.h>

/*Sleep, unused at the moment*/
#include<unistd.h>


#include "hwos_if.h"
#include "queuelist.h"
#include "memory.h"
#include "devicelist.h"

#define LOGPATH "/var/hwos.log"
FILE *logfile;
req_msg_t *recv;
resp_msg_t *send;

void cleanUp();
void signal_handler(int sig);

static void intToChar(char buf[], int value, int pos){
	int i,mask,tmp;
	mask = 0x000000FF;	
	for(i=0; i< 4; i++){
		tmp = (value & mask) >>(8*i);
		if(i==3) tmp = tmp & 0x000000FF;		;		
		buf[i+pos] =tmp;				
		mask = (mask <<8);
	}

}

static int charToInt(char buf[], int pos){
	int res,i,mask;
	mask = 0x000000FF;	
	res =0;
	for(i=0; i< 4; i++){
		res = res | ((buf[pos+i]<< 8*i) & mask);
		mask = (mask <<8);	
	}
	return res;
}



void signal_handler(int sig) {
 
    switch(sig) {
        case SIGHUP:
            //
            break;
        case SIGTERM:
            cleanUp(); 
			break;
        default:
           
            break;
    }
}

void cleanUp(){	
	if(logfile != 0){
		fprintf(logfile,"%s","HWOS DEAMON terminated\n");	
		fclose(logfile);
	}
	if(recv != 0) free(recv);
}


int main(void) {
       
		int ret,i;
		key_t key; /*Key to be passed to msgget()*/ 
		int msgflg;/*Msgflg to be passed to msgget()*/ 
		int msqid; /*Return value from msgget()*/ 
	
		int tmp_msqid;
	
		signal(SIGHUP, signal_handler);
		signal(SIGTERM, signal_handler);
		signal(SIGINT, signal_handler);
		signal(SIGQUIT, signal_handler);

        /* Our process ID and Session ID */
        pid_t pid, sid;
        
        /* Fork off the parent process */
        pid = fork();
        if (pid < 0) {
                exit(EXIT_FAILURE);
        }
        /* If we got a good PID, then
           we can exit the parent process. */
        if (pid > 0) {
                exit(EXIT_SUCCESS);
        }

        /* Change the file mode mask */
        umask(0);
                
        /* Open any logs here */        
        logfile = fopen(LOGPATH,"a+"); 

	        
        /* Create a new SID for the child process */
        sid = setsid();
        if (sid < 0) {
               // fprintf(logfile,"%s","SID failure, exiting...\n");
				cleanUp();
                exit(EXIT_FAILURE);
        }
        

        
        /* Change the current working directory */
        if ((chdir("/")) < 0) {
                //fprintf(logfile,"%s","directory failure, exiting...\n");
				cleanUp();                
				exit(EXIT_FAILURE);
        }
        
        /* Close out the standard file descriptors */
        close(STDIN_FILENO);
        close(STDOUT_FILENO);
        close(STDERR_FILENO);
        
        /* Daemon-specific initialization goes here */
		//fprintf(logfile,"%s","Creating message queue...\n");

		key = 1234;
		msgflg= IPC_CREAT | 0666;
		
		if ((msqid = msgget(key, msgflg)) == -1){
			//fprintf(logfile,"message queue id: %d \n",msqid);
			cleanUp();    		
			return -1;
		} 
		else fprintf(logfile,"message queue created\n");
		

		/*Allocate memory for message buffer*/
		recv = malloc(sizeof(req_msg_t));
		send = malloc(sizeof(resp_msg_t));
		send->type = 11;	

		/*Initialize a dynamic mermoy module with 256 lines*/
		dmemInit(256);
		
		//fprintf(logfile,"DAEMON successfully configured...\n");

		/*Message queue initialization*/
		key = 1234;
		msgflg= IPC_CREAT | 0666;

		if ((msqid = msgget(key, msgflg)) == -1){
			fprintf(logfile,"message queue id: %d \n",msqid);
			cleanUp();    		
			return -1;
		} 
		else fprintf(logfile,"message queue created\n");
		

		/*Allocate memory for message buffer*/
		recv = malloc(sizeof(req_msg_t));
		fprintf(logfile,"%s","DAEMON successfully configured...\n");
		fclose(logfile);
         /*Service loop */
        while (1) {
		  /*Blocking read call*/
           ret = msgrcv(msqid, recv, sizeof(req_msg_t), 12,0);		          
		   if(ret != -1){
				switch(recv->command){	
					//If brackets for this case statement is removed the GCC compiler freaks out 
					case CMD_ALLOC:	{				
						//printf("read data\n");						
						int *base_addr = malloc(sizeof(int));						
						ret = allocate(charToInt(recv->data,0),base_addr);					
						printf("ret: %d\n",ret);							
						if(ret != -1) {
							printDescriptors();						
							send->buf = *base_addr;							
						}
						free(base_addr);
						send->retval =ret;		
						break;
						}
					case CMD_EXEC:
						instrWrite(0,recv->data,recv->bytes);
						send->retval =4;				
						break;		
					case CMD_FREE:
						freeMem(charToInt(recv->data,0));	
						send->retval =4;			
						printDescriptors();			
						break;		
					case CMD_LDDEV:	
							send->retval =0;			
							break;
					case CMD_RMDEV:
							send->retval =0;
							break;				
					case CMD_RMQUE:
						if ((tmp_msqid = msgget(recv->sender_id, msgflg)) == -1){
							fprintf(logfile,"Queue register failed: %d \n",msqid);
						} 
						else{
							/*Register queueid for future use*/
							insertQueue(recv->sender_id,tmp_msqid);							
							send->retval =0;
						}						
						break;
					case CMD_READ:
						//fprintf(logfile,"read data\n");					
						dataRead(0,send->data,recv->addr,recv->bytes);						
						send->retval =1;			
						break;					
					case CMD_UMQUE:
						//fprintf(logfile,"unregister message queue\n");						
						removeQueue(charToInt(recv->data,0));
						break;
					case CMD_WRITE:
						//fprintf(logfile,"write data\n");
						dataWrite(0,recv->data,recv->addr,recv->bytes);						
						send->retval =2;			
						break;
					 default:
						break;
				}		   
				/*send response to sender*/
				ret = msgsnd(getQueueId(recv->sender_id),send, sizeof(resp_msg_t),0);		
			}
        }
   exit(EXIT_SUCCESS);
}

