/****************************************************************
*File			: pools.c
*Author			: Vegard Endresen
*Description	: Dynamic list of pools for BRAM memory allocation. 
*****************************************************************/
#include<stdio.h>
#include<stdlib.h>
#include "pools.h"

/*
*FUNCTIONS
*/
void insertPool(int id,int base_addr,int size);
void removePool(int id);

int  getPoolById(int id, pool_t ** pool);
int  getPoolByAddr(int addr, pool_t ** pool);
void setPool(int id,int id_,int base_addr_, int size_);
void setId(int id,int id_);
void clearAll();

/*number of elements in structure*/
int cnt =0;
/*first element in pool list*/
pool_t *first,*last;



/*
*Copy memory pool descriptor to argument
*/
int getPoolById(int id,pool_t **pool){
	
	if(cnt==0) return -1;

	*pool = first;	
	if((*pool)->id == id) return 0;
	while(1){
		//last item
		if((*pool)->next ==0) return -1;	   

		*pool = (*pool)->next;
		if((*pool)-> id ==id) return 0;	
	}	

}

int getPoolByAddr(int addr,pool_t **pool){		
	if(cnt==0) return -1;		
	*pool = first;	
	
	if((*pool)->base_addr == addr) return 0;
	
	while(1){
		//last item
		if((*pool)->next ==0) return -1;	   
		printf("cnt %d\n",cnt);		
		*pool = (*pool)->next;
		int diff = (*pool)->base_addr - addr;
		if(diff>=0 && diff< (*pool)->size) return 0;	
	}	

}


/*
*Insert pool at end of list
*/
void insertPool(int id,int base_addr,int size){
	pool_t *pool = malloc(sizeof(pool_t));
	pool->id =id;
	pool->base_addr = base_addr;
	pool->size = size;
	pool->next =0;
	//first
	if(cnt ==0){
		first = pool;
		last  = pool; 
		pool->prev =0;
	}	
	else{
		last->next = pool;
		pool->prev = last;
		last = pool;	
	}
	cnt++;
}

/*
*Modifies pool id to id_
*/
void setId(int id,int id_){
	int ret;
	pool_t **pool = malloc(sizeof(int));	
	ret = getPoolById(id,pool);
	
	if(ret !=-1) pool[0]->id = id_;
	free(pool);
}

/*
*Modifies pool fields
*/
void setPool(int id,int id_,int base_addr_,int size_){
	int ret;
	pool_t **pool = malloc(sizeof(int));	
	ret = getPoolById(id,pool);
	if(ret !=-1){
		pool[0]->id = id_;
		pool[0]->base_addr = base_addr_;	
		pool[0]->size = size_;	
	}
	free(pool);
}

/*
*Removes pool element where pool->id == id 
*/
void removePool(int id){	
	int ret;
	pool_t **search_pointer;
	pool_t *pool;

	search_pointer = malloc(sizeof(int));
	ret = getPoolById(id,search_pointer);
	if(ret == -1) return;
	pool = *search_pointer;
	free(search_pointer);

	//only element left
	if(cnt ==1){
		free(first);
		last =0;
		cnt --;
		return;
	}
	//first element
	if(pool->prev ==0){
		pool  = pool->next;
		first = pool;
		free(pool->prev);
		pool->prev=0;			
	}
	//last element
	else if(pool->next ==0){
		pool  = pool->prev;
		last = pool;
		free(pool->next);
		pool->next=0;	
	}
	//in-between
	else{
		pool->prev->next = pool->next;
		pool = pool->next;
		pool_t *tmp = pool->prev;
		pool->prev = pool->prev->prev;
		free(tmp);
	}

	cnt--;
}

void clearAll(){
	pool_t *it = first;
	while(it->next !=0){
		it = it->next;
		removePool(it->prev->id);
	}
	removePool(it->id);
}

pool_t *getFirst(){
	return first; 
}


