/****************************************************************************
*File			: queuelist.c
*Author			: Vegard Endresen
*Description	: Hashmap<int,int> for (message queue key, message queue id)
*****************************************************************************/
#include<stdio.h>
#include<stdlib.h>
#include "queuelist.h"

/*
*FUNCTIONS
*/
int  getNumQueues();
int  getQueueByKey(int key, queue_t ** queue);
void insertQueue(int key,int id);
void removeQueue(int key);
void clearqueues();
queue_t * getFirstQueue();



//number of elements in structure
int queue_cnt =0;
//first element in pool list
queue_t *firstq,*lastq;

int getNumQueues(){
	return queue_cnt;
}

/*
*Copy memory pool descriptor to argument
*/
int getQueueByKey(int key,queue_t **queue){
	if(queue_cnt==0) return -1;
	*queue = firstq;	
	if((*queue)->key == key) return 0;
	while(1){
		//last item
		if((*queue)->next ==0) return -1;	   

		*queue = (*queue)->next;
		if((*queue)-> key == key) return 0;	
	}	

}

int getQueueId(int key){
	queue_t *it;
	if(queue_cnt==0) return -1;

	it = firstq;	
	if(it->key == key) return it->id;
	while(1){
		//last item
		if(it->next ==0) return -1;	   

		it = it->next;
		if(it->key == key) return it->id;	
	}	
}


/*
*Insert pool at end of list
*/
void insertQueue(int key,int id){
	queue_t *queue = malloc(sizeof(queue_t));
	queue->key = key;
	queue->id =id;
	//first
	if(queue_cnt ==0){
		firstq = queue;
		lastq  = queue; 
		queue->prev =0;
	}	
	else{
		lastq->next = queue;
		queue->prev = lastq;
		lastq = queue;	
	}
	queue_cnt++;
}


/*
*Removes pool element where pool->id == id 
*/
void removeQueue(int key){	
	int ret;
	queue_t **search_pointer;
	queue_t *queue;

	search_pointer = malloc(sizeof(int));
	ret = getQueueByKey(key,search_pointer);
	if(ret == -1) return;
	queue = *search_pointer;
	free(search_pointer);

	//only element left
	if(queue_cnt ==1){
		free(firstq);
		lastq =0;
		queue_cnt --;
		return;
	}
	//first element
	if(queue->prev ==0){
		queue  = queue->next;
		firstq = queue;
		free(queue->prev);
		queue->prev=0;			
	}
	//last element
	else if(queue->next ==0){
		queue  = queue->prev;
		lastq = queue;
		free(queue->next);
		queue->next=0;	
	}
	//in-between
	else{
		queue->prev->next = queue->next;
		queue = queue->next;
		queue_t *tmp = queue->prev;
		queue->prev = queue->prev->prev;
		free(tmp);
	}

	queue_cnt--;
}

void clearQueues(){
	queue_t *it = firstq;
	while(it->next !=0){
		it = it->next;
		removeQueue(it->prev->key);
	}
	removeQueue(it->key);
}

queue_t *getFirstQueue(){
	return firstq; 
}

