///////////////////////////////////////////////////////////////////////////
//File			: mult.c
//Description	: Kernel module for id cache debug
//Notes			: --
//Author		: Vegard Endresen		
///////////////////////////////////////////////////////////////////////////

#include <linux/init.h>
#include <linux/module.h>
#include <asm/io.h>

#define byte 8
#define alt_byte 4
#define reg_base_addr 0x81000400
#define mem_base_addr 0x81000000
#define byte 8

MODULE_LICENSE("Dual BSD/GPL");

void *reg_base;
void *mem_base;

/*Transform PowerPC endian to linear bit string and vice versa*/
int transform(int word){
	int mask,part,res,i;
	int shifts[] = {3,1,-1,-3};
	mask =0x000000FF;
	res =0;
	for(i=0; i<4; i++){
		if(shifts[i]>0)	part = (word &mask ) << (shifts[i]*byte);
		else part = (word &mask ) >> (-shifts[i]*byte);
		if(i==3) part = part & 	0x000000FF;	
		res = res | part;		
		mask = (mask <<	byte);
	}
	return res;
}


static int load(void){

	unsigned int i,instr;

	printk(KERN_ALERT "Module loaded\n");
	printk(KERN_ALERT "Remapping address space\n");

	/*Remap software register space*/
	reg_base = ioremap(reg_base_addr,256);
	if (reg_base == 0){
		printk(KERN_ALERT "BRAM remap failure\n");
		return 0;
	}
	/*Remap BRAM space*/
	mem_base = ioremap(mem_base_addr,1024);
	if (mem_base == 0){
		printk(KERN_ALERT "BRAM remap failure\n");
		return 0;
	}
	
	/*Blank 40 first entries of BRAM*/
	for(i=0; i< 40; i++){
			writel(0,mem_base +i*4);	
	}
	/*Start synchronization*/
	writel(transform(0xF000F000),mem_base);
	
	printk("writing program ...\n");
	
	writel(transform(0x8003000C),mem_base+4);	
	writel(transform(0x8003000D),mem_base+8);
	writel(transform(0x8003000E),mem_base+12);
	writel(transform(0x8004000F),mem_base+16);
	writel(transform(0x0010C000),mem_base+20);
	writel(transform(0x40128013),mem_base+24); 
	writel(transform(0x80130011),mem_base+28);
	writel(transform(0x00004013),mem_base+32);
	writel(transform(0xF000F000),mem_base+36);
	writel(transform(0xF000F000),mem_base+40);
	writel(transform(0xA2000000),mem_base+44); 
	writel(transform(0x5BF134F0),mem_base+48);
	writel(transform(0xABCDEF12),mem_base+52);
	writel(transform(0x00ABCDEF),mem_base+56);	
	writel(transform(0x0000004C),mem_base+60);
	writel(transform(0x12345678),mem_base+64); 
	writel(transform(0x12345678),mem_base+68);
	
	/*Specifies an instruction depth of 11*/
	writel(transform(0x8A000000),reg_base);
	
	int timer =0;
	unsigned int tmp =0;
	printk("executing ...\n");
	/*Start timer*/
	writel(transform(0x4A000000),reg_base);
	/*Overwrite sync OP at [0]*/	
	writel(transform(0x8003000B),mem_base);
	
	/*Wait for 1 at conf_register[0]*/
	while(1==1){
		tmp = transform(readl(reg_base));
		if((tmp & 00000001) ==1){
			writel(transform(0x14000000),reg_base);		
			printk("count %x\n",transform(readl(reg_base+4)));	
			break;		
		}
	}
	/*Print BRAM content*/
	for(i=10; i<24; i++){
		printk(KERN_ALERT "bram[%d]: %x\n",i,transform(readl(mem_base+i*4)));
	}
	
	return 0;
}

static void unload(void){
	iounmap(reg_base);
	iounmap(mem_base);
	printk(KERN_ALERT "Module unloaded.\n");
}

module_init(load);
module_exit(unload);

