-------------------------------------------------------------------------------
--
-- Title       : comparator
-- Design      : comparator
-- Author      : Vegard
-- Company     : NTNU	  
-- File        : comparator.vhd
-- Description : 
-------------------------------------------------------------------------------

library IEEE;
use IEEE.STD_LOGIC_1164.all;
use IEEE.std_logic_unsigned.all;	  
use IEEE.std_logic_arith.all;


entity comparator is
	generic(
	constant C_WIDTH : natural :=32; 
	constant C_LOG_WIDTH : natural :=5
	);
	port(												
		Clk		   : in std_logic;
		Reset	   	: in std_logic; 
		Start	   	: in std_logic;
		Operand_a  : in std_logic_vector(C_WIDTH-1 downto 0);
		Operand_b  : in std_logic_vector(C_WIDTH-1 downto 0);
		Comparison : inout std_logic_vector(1 downto 0);
		Complete   : out std_logic
		);
	
	
end comparator;


architecture comparator of comparator is

type STATE is (IDLE,COUNTING);

signal cstate,nstate : STATE;

signal counter	 	: std_logic_vector(C_LOG_WIDTH-1 downto 0);
signal count,break  : std_logic;

signal bit_a,bit_b  : std_logic;

begin		  
	
	NSTATE_PROC : process(cstate,Start,break) is
	begin
		case cstate is
			when IDLE =>
				if Start ='1' then
					nstate <= COUNTING;
				else 
					nstate <= IDLE;
				end if;
			WHEN COUNTING =>
				if break ='1' then
					nstate <= IDLE;
				else
					nstate <= COUNTING;
				end if;				   
		end case;
	end process NSTATE_PROC;
	
	CSTATE_PROC : process(clk) is 
	begin
		if clk'event and  clk='1' then
			if reset ='1' then
				cstate <= IDLE;
			else
				cstate <= nstate;
			end if;
		end if;
	end process CSTATE_PROC;
	
	COUNTER_PROC : process(clk) is
	begin
		if clk'event and clk ='1' then
			if reset ='1' then
				counter <= (others => '0');
			elsif count ='1' then
				counter <= counter +1;
			end if;
		end if;
	end process COUNTER_PROC;
	
	COUNT_PROC : process(cstate) is
	begin
		if cstate = COUNTING then
			count <='1';
		else
			count <='0';
		end if;
	end process COUNT_PROC;
	
	
	BIT_PROC : process (counter) is
	begin
		bit_a <= Operand_a(CONV_INTEGER(counter));
		bit_b <= Operand_b(CONV_INTEGER(counter));
		
	end process BIT_PROC;
	
	COMPARATOR_PROC : process(cstate,bit_a,bit_b) is
	begin
		if bit_a /= bit_b then
			if bit_a ='0' and bit_b ='1' then
				Comparison <="01"; -- Less than
			else
				Comparison <="10"; -- More than		
			end if;
		else
			Comparison <="00";	   -- Equal
		end if;
	end process COMPARATOR_PROC;
	
	COMPLETE_PROC : process(break,cstate)is
	begin						  
		if cstate = COUNTING and break ='1' then
			complete <='1';
		else
			complete <='0';
		end if;
	end process COMPLETE_PROC;
	
	BREAK_PROC : process(Comparison,counter) is
	begin
		if comparison(0) = '1' or comparison(1) = '1' or counter = C_WIDTH-1 then
			break <= '1';
		else
			break <= '0';
		end if;
	end process BREAK_PROC;
	
	
end comparator;
