-------------------------------------------------------------------------------
--
-- Title       : comparator_module
-- Design      : comparator
-- Author      : Vegard
-- Company     : NTNU
-- File        : comparator_module.vhd
-- Description : 
-------------------------------------------------------------------------------


library IEEE;
use IEEE.STD_LOGIC_1164.all;

entity comparator_module is	
	generic(
		OPERAND_SIZE      	: natural	:=32;
		SHIFT_SIZE			: natural   :=8;
		LOG_OPERAND_SIZE	: natural   :=5
		);
	-- Reconfigurable interface
	port(
		clk			: in 	std_logic;
		reset		: in    std_logic;
		run			: in 	std_logic; 
		addr		: in    std_logic;
		zerofill	: in    std_logic;
		shift_i 	: in 	std_logic_vector(SHIFT_SIZE-1 downto 0);
		shift_g		: in 	std_logic;
		shift_o 	: out 	std_logic_vector(SHIFT_SIZE-1 downto 0);
		finish		: out	std_logic 
	);
end comparator_module;


architecture comparator_module of comparator_module is

signal operand_a : std_logic_vector(OPERAND_SIZE-1 downto 0);
signal operand_b : std_logic_vector(OPERAND_SIZE-1 downto 0);

signal zeros : std_logic_vector(OPERAND_SIZE-1 downto 0);

signal shift_a,shift_b : std_logic;
signal sel : std_logic_vector(1 downto 0);

signal mux_i : std_logic_vector(2*SHIFT_SIZE-1 downto 0);

signal comparison : std_logic_vector(1 downto 0);
signal comparison_ex : std_logic_vector(OPERAND_SIZE-1 downto 0);
signal finish_i : std_logic;

begin					
	
		shift_a <= shift_g and sel(0);
		shift_b  <= shift_g and sel(1); 
	
		zeros <= (others =>'0');
		
		comparison_ex(OPERAND_SIZE-1 downto 2)<= zeros(OPERAND_SIZE-1 downto 2);
		comparison_ex(1 downto 0) <= comparison;
		
		finish <= finish_i;
		
		
		ADDR_DECODER: entity addr_decoder
			port map(
				addr => addr,
				output => sel
			);
		
		MUX : entity addr_mux
			port map(
				input => mux_i,
				addr  => addr,
				output => shift_o	
			);
			
	
	
		REG_A : entity register_macro
			generic map(
				C_SIZE 		=> 8,
				C_NUM_REG  => 4
			)
			port map(
				clk	 	=> clk,
				shift 	=> shift_a,
				write 	=> zeros(0),
				shift_i => shift_i,
				data_i 	=> zeros,
				data_o 	=> operand_a,  -- open
				shift_o	=> mux_i(SHIFT_SIZE-1 downto 0)
			);
		
		
		REG_B : entity register_macro
			generic map(
				C_SIZE 		=> 8,
				C_NUM_REG  => 4
			)
			port map(
				clk 	=> clk,
				shift 	=> shift_b,
				shift_i => shift_i,
				write   => finish_i,
				shift_o => mux_i(2*SHIFT_SIZE-1 downto SHIFT_SIZE),
				data_i 	=> comparison_ex,
				data_o	=> operand_b
			);		  
			
		COMP : entity comparator
			generic map(
				C_WIDTH => OPERAND_SIZE,
				C_LOG_WIDTH => LOG_OPERAND_SIZE
			)
			
			port map(
				Clk => clk,
				Reset => reset,
				Start => run,
				Operand_a => operand_a,
				Operand_b => operand_b,
				Comparison => comparison,
				Complete => finish_i
			);
	

end comparator_module;
